from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DATA_SOURCE_TYPE_BATCH: DataSourceType
DATA_SOURCE_TYPE_DATASET_GENERATION: DataSourceType
DATA_SOURCE_TYPE_DELETION: DataSourceType
DATA_SOURCE_TYPE_FEATURE_EXPORT: DataSourceType
DATA_SOURCE_TYPE_INGEST: DataSourceType
DATA_SOURCE_TYPE_STREAM: DataSourceType
DATA_SOURCE_TYPE_UNSPECIFIED: DataSourceType
DESCRIPTOR: _descriptor.FileDescriptor
MATERIALIZATION_STATUS_STATE_DRAINED: MaterializationStatusState
MATERIALIZATION_STATUS_STATE_ERROR: MaterializationStatusState
MATERIALIZATION_STATUS_STATE_MANUALLY_CANCELLED: MaterializationStatusState
MATERIALIZATION_STATUS_STATE_MANUAL_CANCELLATION_REQUESTED: MaterializationStatusState
MATERIALIZATION_STATUS_STATE_PENDING: MaterializationStatusState
MATERIALIZATION_STATUS_STATE_RUNNING: MaterializationStatusState
MATERIALIZATION_STATUS_STATE_SCHEDULED: MaterializationStatusState
MATERIALIZATION_STATUS_STATE_SUCCESS: MaterializationStatusState
MATERIALIZATION_STATUS_STATE_UNSPECIFIED: MaterializationStatusState

class AttemptConsumptionInfo(_message.Message):
    __slots__ = ["job_duration", "offline_rows_written", "online_rows_written"]
    JOB_DURATION_FIELD_NUMBER: _ClassVar[int]
    OFFLINE_ROWS_WRITTEN_FIELD_NUMBER: _ClassVar[int]
    ONLINE_ROWS_WRITTEN_FIELD_NUMBER: _ClassVar[int]
    job_duration: _duration_pb2.Duration
    offline_rows_written: int
    online_rows_written: int
    def __init__(self, online_rows_written: _Optional[int] = ..., offline_rows_written: _Optional[int] = ..., job_duration: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...) -> None: ...

class MaterializationAttemptStatus(_message.Message):
    __slots__ = ["allow_cancel", "allow_forced_retry", "allow_overwrite_retry", "allow_restart", "attempt_consumption", "attempt_created_at", "attempt_number", "data_source_type", "duration", "feature_end_time", "feature_start_time", "is_permanent_failure", "materialization_attempt_id", "materialization_state", "materialization_task_created_at", "materialization_task_id", "progress", "retry_time", "run_page_url", "spark_cluster_environment_version", "spot_instance_failure", "state_message", "tecton_managed_attempt_id", "tecton_runtime_version", "termination_reason", "window_end_time", "window_start_time", "write_to_offline_feature_store", "write_to_online_feature_store"]
    ALLOW_CANCEL_FIELD_NUMBER: _ClassVar[int]
    ALLOW_FORCED_RETRY_FIELD_NUMBER: _ClassVar[int]
    ALLOW_OVERWRITE_RETRY_FIELD_NUMBER: _ClassVar[int]
    ALLOW_RESTART_FIELD_NUMBER: _ClassVar[int]
    ATTEMPT_CONSUMPTION_FIELD_NUMBER: _ClassVar[int]
    ATTEMPT_CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    ATTEMPT_NUMBER_FIELD_NUMBER: _ClassVar[int]
    DATA_SOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    DURATION_FIELD_NUMBER: _ClassVar[int]
    FEATURE_END_TIME_FIELD_NUMBER: _ClassVar[int]
    FEATURE_START_TIME_FIELD_NUMBER: _ClassVar[int]
    IS_PERMANENT_FAILURE_FIELD_NUMBER: _ClassVar[int]
    MATERIALIZATION_ATTEMPT_ID_FIELD_NUMBER: _ClassVar[int]
    MATERIALIZATION_STATE_FIELD_NUMBER: _ClassVar[int]
    MATERIALIZATION_TASK_CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    MATERIALIZATION_TASK_ID_FIELD_NUMBER: _ClassVar[int]
    PROGRESS_FIELD_NUMBER: _ClassVar[int]
    RETRY_TIME_FIELD_NUMBER: _ClassVar[int]
    RUN_PAGE_URL_FIELD_NUMBER: _ClassVar[int]
    SPARK_CLUSTER_ENVIRONMENT_VERSION_FIELD_NUMBER: _ClassVar[int]
    SPOT_INSTANCE_FAILURE_FIELD_NUMBER: _ClassVar[int]
    STATE_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    TECTON_MANAGED_ATTEMPT_ID_FIELD_NUMBER: _ClassVar[int]
    TECTON_RUNTIME_VERSION_FIELD_NUMBER: _ClassVar[int]
    TERMINATION_REASON_FIELD_NUMBER: _ClassVar[int]
    WINDOW_END_TIME_FIELD_NUMBER: _ClassVar[int]
    WINDOW_START_TIME_FIELD_NUMBER: _ClassVar[int]
    WRITE_TO_OFFLINE_FEATURE_STORE_FIELD_NUMBER: _ClassVar[int]
    WRITE_TO_ONLINE_FEATURE_STORE_FIELD_NUMBER: _ClassVar[int]
    allow_cancel: bool
    allow_forced_retry: bool
    allow_overwrite_retry: bool
    allow_restart: bool
    attempt_consumption: AttemptConsumptionInfo
    attempt_created_at: _timestamp_pb2.Timestamp
    attempt_number: int
    data_source_type: DataSourceType
    duration: _duration_pb2.Duration
    feature_end_time: _timestamp_pb2.Timestamp
    feature_start_time: _timestamp_pb2.Timestamp
    is_permanent_failure: bool
    materialization_attempt_id: _id__client_pb2.Id
    materialization_state: MaterializationStatusState
    materialization_task_created_at: _timestamp_pb2.Timestamp
    materialization_task_id: _id__client_pb2.Id
    progress: float
    retry_time: _timestamp_pb2.Timestamp
    run_page_url: str
    spark_cluster_environment_version: int
    spot_instance_failure: bool
    state_message: str
    tecton_managed_attempt_id: _id__client_pb2.Id
    tecton_runtime_version: str
    termination_reason: str
    window_end_time: _timestamp_pb2.Timestamp
    window_start_time: _timestamp_pb2.Timestamp
    write_to_offline_feature_store: bool
    write_to_online_feature_store: bool
    def __init__(self, data_source_type: _Optional[_Union[DataSourceType, str]] = ..., window_start_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., window_end_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., feature_start_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., feature_end_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., materialization_state: _Optional[_Union[MaterializationStatusState, str]] = ..., state_message: _Optional[str] = ..., termination_reason: _Optional[str] = ..., spot_instance_failure: bool = ..., materialization_task_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., materialization_attempt_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., materialization_task_created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., attempt_number: _Optional[int] = ..., spark_cluster_environment_version: _Optional[int] = ..., tecton_runtime_version: _Optional[str] = ..., run_page_url: _Optional[str] = ..., tecton_managed_attempt_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., attempt_consumption: _Optional[_Union[AttemptConsumptionInfo, _Mapping]] = ..., attempt_created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., is_permanent_failure: bool = ..., retry_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., progress: _Optional[float] = ..., duration: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., allow_forced_retry: bool = ..., allow_overwrite_retry: bool = ..., allow_cancel: bool = ..., allow_restart: bool = ..., write_to_online_feature_store: bool = ..., write_to_offline_feature_store: bool = ...) -> None: ...

class MaterializationStatus(_message.Message):
    __slots__ = ["feature_package_id", "materialization_attempts", "schedule_error_message"]
    FEATURE_PACKAGE_ID_FIELD_NUMBER: _ClassVar[int]
    MATERIALIZATION_ATTEMPTS_FIELD_NUMBER: _ClassVar[int]
    SCHEDULE_ERROR_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    feature_package_id: _id__client_pb2.Id
    materialization_attempts: _containers.RepeatedCompositeFieldContainer[MaterializationAttemptStatus]
    schedule_error_message: str
    def __init__(self, feature_package_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., materialization_attempts: _Optional[_Iterable[_Union[MaterializationAttemptStatus, _Mapping]]] = ..., schedule_error_message: _Optional[str] = ...) -> None: ...

class DataSourceType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class MaterializationStatusState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/principal/principal_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.auditlog import metadata__client_pb2 as tecton__proto_dot_auditlog_dot_metadata____client__pb2
from tecton_proto.auth import principal__client_pb2 as tecton__proto_dot_auth_dot_principal____client__pb2
from tecton_proto.auth import resource_role_assignments__client_pb2 as tecton__proto_dot_auth_dot_resource__role__assignments____client__pb2
from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2
from tecton_proto.data import principal_group__client_pb2 as tecton__proto_dot_data_dot_principal__group____client__pb2
from tecton_proto.data import user__client_pb2 as tecton__proto_dot_data_dot_user____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6tecton_proto/principal/principal_service__client.proto\x12\x16tecton_proto.principal\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/auditlog/metadata__client.proto\x1a)tecton_proto/auth/principal__client.proto\x1a\x39tecton_proto/auth/resource_role_assignments__client.proto\x1a\'tecton_proto/auth/service__client.proto\x1a/tecton_proto/data/principal_group__client.proto\x1a$tecton_proto/data/user__client.proto\"\xa0\x01\n\x1b\x43reatePrincipalGroupRequest\x12\x1b\n\x04name\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x04name\x12)\n\x0b\x64\x65scription\x18\x03 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0b\x64\x65scription\x12\x33\n\x11idp_mapping_names\x18\x04 \x03(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0fidpMappingNamesJ\x04\x08\x01\x10\x02\"s\n\x1c\x43reatePrincipalGroupResponse\x12S\n\x0fprincipal_group\x18\x01 \x01(\x0b\x32!.tecton_proto.data.PrincipalGroupB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0eprincipalGroup\"\xb3\x01\n\x1bUpdatePrincipalGroupRequest\x12\x17\n\x02id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x02id\x12\x1b\n\x04name\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x04name\x12)\n\x0b\x64\x65scription\x18\x03 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0b\x64\x65scription\x12\x33\n\x11idp_mapping_names\x18\x04 \x03(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0fidpMappingNames\"s\n\x1cUpdatePrincipalGroupResponse\x12S\n\x0fprincipal_group\x18\x01 \x01(\x0b\x32!.tecton_proto.data.PrincipalGroupB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0eprincipalGroup\"\x12\n\x10ListUsersRequest\"B\n\x11ListUsersResponse\x12-\n\x05users\x18\x01 \x03(\x0b\x32\x17.tecton_proto.data.UserR\x05users\"9\n\x1c\x44\x65letePrincipalGroupsRequest\x12\x19\n\x03ids\x18\x01 \x03(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x03ids\"\x1f\n\x1d\x44\x65letePrincipalGroupsResponse\"N\n\"ListPrincipalGroupsDetailedRequest\x12\x16\n\x06search\x18\x01 \x01(\tR\x06search\x12\x10\n\x03ids\x18\x02 \x03(\tR\x03ids\"m\n#ListPrincipalGroupsDetailedResponse\x12\x46\n\x06groups\x18\x01 \x03(\x0b\x32..tecton_proto.principal.PrincipalGroupDetailedR\x06groups\"\xd8\x03\n\x16PrincipalGroupDetailed\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12 \n\x0b\x64\x65scription\x18\x03 \x01(\tR\x0b\x64\x65scription\x12*\n\x11idp_mapping_names\x18\x04 \x03(\tR\x0fidpMappingNames\x12\x34\n\x16is_membership_editable\x18\x05 \x01(\x08R\x14isMembershipEditable\x12\x39\n\ncreated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12!\n\x0c\x61\x63\x63ount_type\x18\x07 \x01(\tR\x0b\x61\x63\x63ountType\x12\x1f\n\x0bnum_members\x18\x08 \x01(\x05R\nnumMembers\x12O\n\nworkspaces\x18\n \x03(\x0b\x32/.tecton_proto.auth.ResourceAndRoleAssignmentsV2R\nworkspaces\x12@\n\ncreated_by\x18\x0b \x01(\x0b\x32!.tecton_proto.auth.PrincipalBasicR\tcreatedByJ\x04\x08\t\x10\n\"\xac\x01\n\x1f\x41\x64\x64PrincipalGroupMembersRequest\x12\x35\n\x12principal_group_id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x10principalGroupId\x12R\n\x07members\x18\x02 \x03(\x0b\x32/.tecton_proto.principal.AddPrincipalGroupMemberB\x07\xea\x99\xbd\x46\x02\x08\x02R\x07members\"\x97\x01\n\x17\x41\x64\x64PrincipalGroupMember\x12P\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeB\x07\xea\x99\xbd\x46\x02\x08\x02R\rprincipalType\x12*\n\x0cprincipal_id\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0bprincipalId\"\"\n AddPrincipalGroupMembersResponse\"\x95\x01\n ListPrincipalGroupMembersRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x16\n\x06search\x18\x02 \x01(\tR\x06search\x12I\n\x0fprincipal_types\x18\x03 \x03(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\x0eprincipalTypes\"k\n!ListPrincipalGroupMembersResponse\x12\x46\n\x07members\x18\x01 \x03(\x0b\x32,.tecton_proto.principal.PrincipalGroupMemberR\x07members\"\x86\x01\n\x14PrincipalGroupMember\x12?\n\tprincipal\x18\x01 \x01(\x0b\x32!.tecton_proto.auth.PrincipalBasicR\tprincipal\x12-\n\x13is_from_idp_mapping\x18\x02 \x01(\x08R\x10isFromIdpMapping\"\x94\x01\n\"RemovePrincipalGroupMembersRequest\x12\x17\n\x02id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x02id\x12U\n\x07members\x18\x02 \x03(\x0b\x32\x32.tecton_proto.principal.RemovePrincipalGroupMemberB\x07\xea\x99\xbd\x46\x02\x08\x02R\x07members\"\x9a\x01\n\x1aRemovePrincipalGroupMember\x12P\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeB\x07\xea\x99\xbd\x46\x02\x08\x02R\rprincipalType\x12*\n\x0cprincipal_id\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0bprincipalId\"%\n#RemovePrincipalGroupMembersResponse\"\xac\x01\n&ListPrincipalGroupsForPrincipalRequest\x12G\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\rprincipalType\x12!\n\x0cprincipal_id\x18\x02 \x01(\tR\x0bprincipalId\x12\x16\n\x06search\x18\x03 \x01(\tR\x06search\"u\n\'ListPrincipalGroupsForPrincipalResponse\x12J\n\x06groups\x18\x01 \x03(\x0b\x32\x32.tecton_proto.principal.PrincipalGroupForPrincipalR\x06groups\"\x84\x01\n\x1aPrincipalGroupForPrincipal\x12\x37\n\x05group\x18\x01 \x01(\x0b\x32!.tecton_proto.data.PrincipalGroupR\x05group\x12-\n\x13is_from_idp_mapping\x18\x02 \x01(\x08R\x10isFromIdpMapping2\xa2\x0e\n\x10PrincipalService\x12\xd7\x01\n\x14\x43reatePrincipalGroup\x12\x33.tecton_proto.principal.CreatePrincipalGroupRequest\x1a\x34.tecton_proto.principal.CreatePrincipalGroupResponse\"T\xa2\xbc\xe6\xc0\x05,**tecton/organization#create_principal_group\xf2\x99\xbd\x46\x1d\x08\x01\x12\x16\x63reate_principal_group\x1a\x01\x31\x12\xd7\x01\n\x14UpdatePrincipalGroup\x12\x33.tecton_proto.principal.UpdatePrincipalGroupRequest\x1a\x34.tecton_proto.principal.UpdatePrincipalGroupResponse\"T\xa2\xbc\xe6\xc0\x05,**tecton/organization#manage_principal_group\xf2\x99\xbd\x46\x1d\x08\x01\x12\x16update_principal_group\x1a\x01\x31\x12\xdb\x01\n\x15\x44\x65letePrincipalGroups\x12\x34.tecton_proto.principal.DeletePrincipalGroupsRequest\x1a\x35.tecton_proto.principal.DeletePrincipalGroupsResponse\"U\xa2\xbc\xe6\xc0\x05,**tecton/organization#manage_principal_group\xf2\x99\xbd\x46\x1e\x08\x01\x12\x17\x64\x65lete_principal_groups\x1a\x01\x31\x12\xa0\x01\n\x1bListPrincipalGroupsDetailed\x12:.tecton_proto.principal.ListPrincipalGroupsDetailedRequest\x1a;.tecton_proto.principal.ListPrincipalGroupsDetailedResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\xfb\x01\n\x18\x41\x64\x64PrincipalGroupMembers\x12\x37.tecton_proto.principal.AddPrincipalGroupMembersRequest\x1a\x38.tecton_proto.principal.AddPrincipalGroupMembersResponse\"l\xa2\xbc\xe6\xc0\x05?\x1a\x16\x08\x03\x12\x12principal_group_id*%tecton/principal_group#manage_members\xf2\x99\xbd\x46\"\x08\x01\x12\x1b\x61\x64\x64_principal_group_members\x1a\x01\x31\x12\xc5\x01\n\x19ListPrincipalGroupMembers\x12\x38.tecton_proto.principal.ListPrincipalGroupMembersRequest\x1a\x39.tecton_proto.principal.ListPrincipalGroupMembersResponse\"3\xa2\xbc\xe6\xc0\x05-\x1a\x06\x08\x03\x12\x02id*#tecton/principal_group#list_members\x12j\n\tListUsers\x12(.tecton_proto.principal.ListUsersRequest\x1a).tecton_proto.principal.ListUsersResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\xf7\x01\n\x1bRemovePrincipalGroupMembers\x12:.tecton_proto.principal.RemovePrincipalGroupMembersRequest\x1a;.tecton_proto.principal.RemovePrincipalGroupMembersResponse\"_\xa2\xbc\xe6\xc0\x05/\x1a\x06\x08\x03\x12\x02id*%tecton/principal_group#manage_members\xf2\x99\xbd\x46%\x08\x01\x12\x1eremove_principal_group_members\x1a\x01\x31\x12\xac\x01\n\x1fListPrincipalGroupsForPrincipal\x12>.tecton_proto.principal.ListPrincipalGroupsForPrincipalRequest\x1a?.tecton_proto.principal.ListPrincipalGroupsForPrincipalResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x42\\\n\x14\x63om.tecton.principalB\x15PrincipalServiceProtoP\x01Z+github.com/tecton-ai/tecton_proto/principal')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.principal.principal_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024com.tecton.principalB\025PrincipalServiceProtoP\001Z+github.com/tecton-ai/tecton_proto/principal'
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['name']._options = None
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['name']._serialized_options = b'\352\231\275F\002\010\002'
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['description']._options = None
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['description']._serialized_options = b'\352\231\275F\002\010\002'
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['idp_mapping_names']._options = None
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['idp_mapping_names']._serialized_options = b'\352\231\275F\002\010\002'
  _CREATEPRINCIPALGROUPRESPONSE.fields_by_name['principal_group']._options = None
  _CREATEPRINCIPALGROUPRESPONSE.fields_by_name['principal_group']._serialized_options = b'\352\231\275F\002\010\002'
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['id']._options = None
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['id']._serialized_options = b'\352\231\275F\002\010\002'
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['name']._options = None
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['name']._serialized_options = b'\352\231\275F\002\010\002'
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['description']._options = None
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['description']._serialized_options = b'\352\231\275F\002\010\002'
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['idp_mapping_names']._options = None
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['idp_mapping_names']._serialized_options = b'\352\231\275F\002\010\002'
  _UPDATEPRINCIPALGROUPRESPONSE.fields_by_name['principal_group']._options = None
  _UPDATEPRINCIPALGROUPRESPONSE.fields_by_name['principal_group']._serialized_options = b'\352\231\275F\002\010\002'
  _DELETEPRINCIPALGROUPSREQUEST.fields_by_name['ids']._options = None
  _DELETEPRINCIPALGROUPSREQUEST.fields_by_name['ids']._serialized_options = b'\352\231\275F\002\010\002'
  _ADDPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['principal_group_id']._options = None
  _ADDPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['principal_group_id']._serialized_options = b'\352\231\275F\002\010\002'
  _ADDPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['members']._options = None
  _ADDPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['members']._serialized_options = b'\352\231\275F\002\010\002'
  _ADDPRINCIPALGROUPMEMBER.fields_by_name['principal_type']._options = None
  _ADDPRINCIPALGROUPMEMBER.fields_by_name['principal_type']._serialized_options = b'\352\231\275F\002\010\002'
  _ADDPRINCIPALGROUPMEMBER.fields_by_name['principal_id']._options = None
  _ADDPRINCIPALGROUPMEMBER.fields_by_name['principal_id']._serialized_options = b'\352\231\275F\002\010\002'
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['id']._options = None
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['id']._serialized_options = b'\352\231\275F\002\010\002'
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['members']._options = None
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['members']._serialized_options = b'\352\231\275F\002\010\002'
  _REMOVEPRINCIPALGROUPMEMBER.fields_by_name['principal_type']._options = None
  _REMOVEPRINCIPALGROUPMEMBER.fields_by_name['principal_type']._serialized_options = b'\352\231\275F\002\010\002'
  _REMOVEPRINCIPALGROUPMEMBER.fields_by_name['principal_id']._options = None
  _REMOVEPRINCIPALGROUPMEMBER.fields_by_name['principal_id']._serialized_options = b'\352\231\275F\002\010\002'
  _PRINCIPALSERVICE.methods_by_name['CreatePrincipalGroup']._options = None
  _PRINCIPALSERVICE.methods_by_name['CreatePrincipalGroup']._serialized_options = b'\242\274\346\300\005,**tecton/organization#create_principal_group\362\231\275F\035\010\001\022\026create_principal_group\032\0011'
  _PRINCIPALSERVICE.methods_by_name['UpdatePrincipalGroup']._options = None
  _PRINCIPALSERVICE.methods_by_name['UpdatePrincipalGroup']._serialized_options = b'\242\274\346\300\005,**tecton/organization#manage_principal_group\362\231\275F\035\010\001\022\026update_principal_group\032\0011'
  _PRINCIPALSERVICE.methods_by_name['DeletePrincipalGroups']._options = None
  _PRINCIPALSERVICE.methods_by_name['DeletePrincipalGroups']._serialized_options = b'\242\274\346\300\005,**tecton/organization#manage_principal_group\362\231\275F\036\010\001\022\027delete_principal_groups\032\0011'
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupsDetailed']._options = None
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupsDetailed']._serialized_options = b'\242\274\346\300\005\002@\001'
  _PRINCIPALSERVICE.methods_by_name['AddPrincipalGroupMembers']._options = None
  _PRINCIPALSERVICE.methods_by_name['AddPrincipalGroupMembers']._serialized_options = b'\242\274\346\300\005?\032\026\010\003\022\022principal_group_id*%tecton/principal_group#manage_members\362\231\275F\"\010\001\022\033add_principal_group_members\032\0011'
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupMembers']._options = None
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupMembers']._serialized_options = b'\242\274\346\300\005-\032\006\010\003\022\002id*#tecton/principal_group#list_members'
  _PRINCIPALSERVICE.methods_by_name['ListUsers']._options = None
  _PRINCIPALSERVICE.methods_by_name['ListUsers']._serialized_options = b'\242\274\346\300\005\002@\001'
  _PRINCIPALSERVICE.methods_by_name['RemovePrincipalGroupMembers']._options = None
  _PRINCIPALSERVICE.methods_by_name['RemovePrincipalGroupMembers']._serialized_options = b'\242\274\346\300\005/\032\006\010\003\022\002id*%tecton/principal_group#manage_members\362\231\275F%\010\001\022\036remove_principal_group_members\032\0011'
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupsForPrincipal']._options = None
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupsForPrincipal']._serialized_options = b'\242\274\346\300\005\002@\001'
  _CREATEPRINCIPALGROUPREQUEST._serialized_start=392
  _CREATEPRINCIPALGROUPREQUEST._serialized_end=552
  _CREATEPRINCIPALGROUPRESPONSE._serialized_start=554
  _CREATEPRINCIPALGROUPRESPONSE._serialized_end=669
  _UPDATEPRINCIPALGROUPREQUEST._serialized_start=672
  _UPDATEPRINCIPALGROUPREQUEST._serialized_end=851
  _UPDATEPRINCIPALGROUPRESPONSE._serialized_start=853
  _UPDATEPRINCIPALGROUPRESPONSE._serialized_end=968
  _LISTUSERSREQUEST._serialized_start=970
  _LISTUSERSREQUEST._serialized_end=988
  _LISTUSERSRESPONSE._serialized_start=990
  _LISTUSERSRESPONSE._serialized_end=1056
  _DELETEPRINCIPALGROUPSREQUEST._serialized_start=1058
  _DELETEPRINCIPALGROUPSREQUEST._serialized_end=1115
  _DELETEPRINCIPALGROUPSRESPONSE._serialized_start=1117
  _DELETEPRINCIPALGROUPSRESPONSE._serialized_end=1148
  _LISTPRINCIPALGROUPSDETAILEDREQUEST._serialized_start=1150
  _LISTPRINCIPALGROUPSDETAILEDREQUEST._serialized_end=1228
  _LISTPRINCIPALGROUPSDETAILEDRESPONSE._serialized_start=1230
  _LISTPRINCIPALGROUPSDETAILEDRESPONSE._serialized_end=1339
  _PRINCIPALGROUPDETAILED._serialized_start=1342
  _PRINCIPALGROUPDETAILED._serialized_end=1814
  _ADDPRINCIPALGROUPMEMBERSREQUEST._serialized_start=1817
  _ADDPRINCIPALGROUPMEMBERSREQUEST._serialized_end=1989
  _ADDPRINCIPALGROUPMEMBER._serialized_start=1992
  _ADDPRINCIPALGROUPMEMBER._serialized_end=2143
  _ADDPRINCIPALGROUPMEMBERSRESPONSE._serialized_start=2145
  _ADDPRINCIPALGROUPMEMBERSRESPONSE._serialized_end=2179
  _LISTPRINCIPALGROUPMEMBERSREQUEST._serialized_start=2182
  _LISTPRINCIPALGROUPMEMBERSREQUEST._serialized_end=2331
  _LISTPRINCIPALGROUPMEMBERSRESPONSE._serialized_start=2333
  _LISTPRINCIPALGROUPMEMBERSRESPONSE._serialized_end=2440
  _PRINCIPALGROUPMEMBER._serialized_start=2443
  _PRINCIPALGROUPMEMBER._serialized_end=2577
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST._serialized_start=2580
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST._serialized_end=2728
  _REMOVEPRINCIPALGROUPMEMBER._serialized_start=2731
  _REMOVEPRINCIPALGROUPMEMBER._serialized_end=2885
  _REMOVEPRINCIPALGROUPMEMBERSRESPONSE._serialized_start=2887
  _REMOVEPRINCIPALGROUPMEMBERSRESPONSE._serialized_end=2924
  _LISTPRINCIPALGROUPSFORPRINCIPALREQUEST._serialized_start=2927
  _LISTPRINCIPALGROUPSFORPRINCIPALREQUEST._serialized_end=3099
  _LISTPRINCIPALGROUPSFORPRINCIPALRESPONSE._serialized_start=3101
  _LISTPRINCIPALGROUPSFORPRINCIPALRESPONSE._serialized_end=3218
  _PRINCIPALGROUPFORPRINCIPAL._serialized_start=3221
  _PRINCIPALGROUPFORPRINCIPAL._serialized_end=3353
  _PRINCIPALSERVICE._serialized_start=3356
  _PRINCIPALSERVICE._serialized_end=5182
# @@protoc_insertion_point(module_scope)



from datetime import datetime
from typing import Any
from typing import Mapping
from typing import Optional

import pandas

from tecton_proto.args.transformation__client_pb2 import TransformationMode


REQUEST_TIMESTAMP_FIELD_NAME = "request_timestamp"

VALID_TRANSFORMATION_MODES = {
    TransformationMode.TRANSFORMATION_MODE_PYTHON,
    TransformationMode.TRANSFORMATION_MODE_PANDAS,
    TransformationMode.TRANSFORMATION_MODE_NO_TRANSFORMATION,
}


class RealtimeContext:
    """
    RealtimeContext is a class that is used to pass context metadata such as the request_timestamp
    to the `context` parameter of a Realtime Feature Views.
    """

    _mode: Optional[TransformationMode.ValueType] = None
    _request_timestamp: Optional[datetime] = None
    _row_level_data: Optional[pandas.DataFrame] = None
    _secrets: Optional[Mapping[str, str]] = None
    _resources: Optional[Mapping[str, Any]] = None

    def __init__(
        self,
        secrets: Optional[Mapping[str, str]] = None,
        resources: Optional[Mapping[str, Any]] = None,
        _request_timestamp: Optional[datetime] = None,
        _row_level_data: Optional[pandas.DataFrame] = None,
        _mode: Optional[TransformationMode.ValueType] = None,
    ) -> None:
        """
        Initialize the RealtimeContext object.

        :param secrets: A map that maps the secret name to the resolved secret value.
        :param resources: A map that maps the resource name to the resource returned by the resource provider.
        :param _mode: Realtime Feature View mode. Valid modes: Python, Pandas, No Transformation.
        :param _request_timestamp: The timestamp of the request made to the Tecton Feature Server. Used for Python mode offline and for both modes online.
        :param _row_level_data: The row-level context data for the Realtime Feature View for each row in the events data frame. Only populated in Pandas mode during Offline Retrieval.
        """
        self._mode = _mode
        self._request_timestamp = _request_timestamp
        self._row_level_data = _row_level_data
        self._secrets = secrets
        self._resources = resources

    def __str__(self) -> str:
        return (
            f"RealtimeContext(mode={self._mode}, "
            f"request_timestamp={self._request_timestamp}, "
            f"has_row_level_data={self._row_level_data is not None}, "
            f"secrets={self._secrets}, "
            f"resources={self._resources})"
        )

    def set_mode(self, mode: TransformationMode.ValueType) -> None:
        """
        Set the mode of the Realtime Feature View.

        :param mode: Mode of the Realtime Feature View.
        """
        assert mode in VALID_TRANSFORMATION_MODES, f"Mode {mode} is not a valid transformation mode."
        self._mode = mode

    def set_request_timestamp(self, request_timestamp: datetime) -> None:
        """
        Set the request_timestamp for the Realtime Feature View.

        :param request_timestamp: The timestamp of the request made to the Tecton Feature Server.
        """
        self._request_timestamp = request_timestamp

    @property
    def request_timestamp(self) -> Optional[datetime]:
        """
        The request_timestamp is the timestamp of the request made to the Tecton Feature Server.
        """
        if self._mode not in [
            TransformationMode.TRANSFORMATION_MODE_PYTHON,
            TransformationMode.TRANSFORMATION_MODE_NO_TRANSFORMATION,
        ]:
            str = "The field `request_timestamp` is not available in Realtime Feature Views with Pandas mode. Please use `context.request_timestamp_series` when using Pandas mode."
            raise ValueError(str)

        return self._request_timestamp

    @property
    def request_timestamp_series(self) -> Optional[pandas.Series]:
        """
        The request_timestamp_series is a pandas Series with the request_timestamp as the only element. Each element in the
        Series is the request_timestamp for the corresponding row in the input data.
        """
        if self._mode != TransformationMode.TRANSFORMATION_MODE_PANDAS:
            str = "The field `request_timestamp_series` is only available in Realtime Feature Views with Pandas mode. Please use `context.request_timestamp` when using Python mode or when no transformations are present."
            raise ValueError(str)

        # For online retrieval, we build a Series with the request_timestamp as the only element
        if self._request_timestamp is not None:
            return pandas.Series([self._request_timestamp], name=REQUEST_TIMESTAMP_FIELD_NAME)

        if self._row_level_data is None:
            return None

        return self._row_level_data[REQUEST_TIMESTAMP_FIELD_NAME]

    @property
    def secrets(self) -> Optional[Mapping[str, str]]:
        return self._secrets

    @property
    def resources(self) -> Optional[Mapping[str, Any]]:
        return self._resources



from tecton_proto.canary import type__client_pb2 as _type__client_pb2
from tecton_proto.data import feature_view__client_pb2 as _feature_view__client_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class OnlineStoreMetadataConfiguration(_message.Message):
    __slots__ = ["execution_table_name", "online_store_params"]
    EXECUTION_TABLE_NAME_FIELD_NUMBER: _ClassVar[int]
    ONLINE_STORE_PARAMS_FIELD_NUMBER: _ClassVar[int]
    execution_table_name: str
    online_store_params: _feature_view__client_pb2.OnlineStoreParams
    def __init__(self, execution_table_name: _Optional[str] = ..., online_store_params: _Optional[_Union[_feature_view__client_pb2.OnlineStoreParams, _Mapping]] = ...) -> None: ...

class OnlineStoreWriterConfiguration(_message.Message):
    __slots__ = ["canary_downsample_factor", "canary_id", "canary_table_name", "canary_type", "data_table_name", "online_store_params", "status_table_name"]
    CANARY_DOWNSAMPLE_FACTOR_FIELD_NUMBER: _ClassVar[int]
    CANARY_ID_FIELD_NUMBER: _ClassVar[int]
    CANARY_TABLE_NAME_FIELD_NUMBER: _ClassVar[int]
    CANARY_TYPE_FIELD_NUMBER: _ClassVar[int]
    DATA_TABLE_NAME_FIELD_NUMBER: _ClassVar[int]
    ONLINE_STORE_PARAMS_FIELD_NUMBER: _ClassVar[int]
    STATUS_TABLE_NAME_FIELD_NUMBER: _ClassVar[int]
    canary_downsample_factor: int
    canary_id: str
    canary_table_name: str
    canary_type: _type__client_pb2.CanaryType
    data_table_name: str
    online_store_params: _feature_view__client_pb2.OnlineStoreParams
    status_table_name: str
    def __init__(self, status_table_name: _Optional[str] = ..., data_table_name: _Optional[str] = ..., canary_table_name: _Optional[str] = ..., canary_type: _Optional[_Union[_type__client_pb2.CanaryType, str]] = ..., canary_id: _Optional[str] = ..., canary_downsample_factor: _Optional[int] = ..., online_store_params: _Optional[_Union[_feature_view__client_pb2.OnlineStoreParams, _Mapping]] = ...) -> None: ...



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class GetUserByIdResponse(_message.Message):
    __slots__ = ["display_name", "user_name"]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    USER_NAME_FIELD_NUMBER: _ClassVar[int]
    display_name: str
    user_name: str
    def __init__(self, user_name: _Optional[str] = ..., display_name: _Optional[str] = ...) -> None: ...



from tecton_proto.common import pair__client_pb2 as _pair__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor
TEMPORAL_FEATURE_VIEW: CanaryFeatureViewType
UNDEFINED: CanaryFeatureViewType
WINDOW_AGGREGATE_FEATURE_VIEW: CanaryFeatureViewType

class CanaryUpdate(_message.Message):
    __slots__ = ["condition_expression", "dynamo_expression_attribute_names", "dynamo_expression_attribute_values", "feature_view_type", "is_status_table", "serialized_items", "table_name", "time"]
    CONDITION_EXPRESSION_FIELD_NUMBER: _ClassVar[int]
    DYNAMO_EXPRESSION_ATTRIBUTE_NAMES_FIELD_NUMBER: _ClassVar[int]
    DYNAMO_EXPRESSION_ATTRIBUTE_VALUES_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_TYPE_FIELD_NUMBER: _ClassVar[int]
    IS_STATUS_TABLE_FIELD_NUMBER: _ClassVar[int]
    SERIALIZED_ITEMS_FIELD_NUMBER: _ClassVar[int]
    TABLE_NAME_FIELD_NUMBER: _ClassVar[int]
    TIME_FIELD_NUMBER: _ClassVar[int]
    condition_expression: str
    dynamo_expression_attribute_names: _containers.RepeatedCompositeFieldContainer[_pair__client_pb2.Pair]
    dynamo_expression_attribute_values: _containers.RepeatedCompositeFieldContainer[_pair__client_pb2.Pair]
    feature_view_type: CanaryFeatureViewType
    is_status_table: bool
    serialized_items: _containers.RepeatedCompositeFieldContainer[_pair__client_pb2.Pair]
    table_name: str
    time: str
    def __init__(self, table_name: _Optional[str] = ..., serialized_items: _Optional[_Iterable[_Union[_pair__client_pb2.Pair, _Mapping]]] = ..., condition_expression: _Optional[str] = ..., dynamo_expression_attribute_names: _Optional[_Iterable[_Union[_pair__client_pb2.Pair, _Mapping]]] = ..., dynamo_expression_attribute_values: _Optional[_Iterable[_Union[_pair__client_pb2.Pair, _Mapping]]] = ..., time: _Optional[str] = ..., is_status_table: bool = ..., feature_view_type: _Optional[_Union[CanaryFeatureViewType, str]] = ...) -> None: ...

class CanaryFeatureViewType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from __future__ import annotations

from dataclasses import dataclass
from types import MappingProxyType
from typing import Any
from typing import Callable
from typing import List
from typing import Mapping
from typing import Optional

import numpy
import pyarrow

from tecton_core import schema
from tecton_core.embeddings.model_container import _CONTEXT_PARAM
from tecton_core.embeddings.model_container import CustomModelContainerBase
from tecton_core.embeddings.model_container import pyarrow_batch_to_numpy_dict
from tecton_core.embeddings.model_container import tecton_type_to_pyarrow
from tecton_core.errors import TectonValidationError


_PREDICT_FN = Optional[Callable[[Mapping[str, numpy.ndarray], _CONTEXT_PARAM], numpy.ndarray]]
_PREDICT_FN_NAME = "predict"


class PythonModelContainer(CustomModelContainerBase):
    _predict: _PREDICT_FN = None

    def load(self):
        model_module = super().load()
        self._predict = getattr(model_module, _PREDICT_FN_NAME, None)
        self._validate_python_model()

    def predict(self, input: pyarrow.RecordBatch) -> numpy.ndarray:
        numpy_dict = pyarrow_batch_to_numpy_dict(input)
        output = self._predict(numpy_dict, self._context)
        if not isinstance(output, numpy.ndarray):
            msg = "Model output from predict should be a numpy array"
            raise TypeError(msg)
        return output

    def _validate_python_model(self):
        if self._predict is None:
            msg = "Model file must have a `predict` function with signature `predict(inputs, context)`."
            raise TectonValidationError(msg)


def python_model_inference(
    batch: pyarrow.RecordBatch,
    model: PythonModelContainer,
    input_columns: List[schema.Column],
    output_column: schema.Column,
    model_input_schema: List[schema.Column],
) -> pyarrow.RecordBatch:
    input_batch = batch.select([col.name for col in input_columns])
    model_input_names = [col.name for col in model_input_schema]

    # Rename input columns to model input schema column names.
    # TODO(jiadong): we can use pyarrow built-in `rename_columns` when we upgrade pyarrow to 16.0+.
    input_columns = [input_batch.column(i) for i in range(input_batch.num_columns)]
    input_batch = pyarrow.RecordBatch.from_arrays(input_columns, model_input_names)

    output = _python_model_predict(model=model, input_batch=input_batch)
    output = _convert_numpy_to_arrow(output, batch.num_rows, tecton_type_to_pyarrow(output_column.dtype))

    batch_with_results = pyarrow.RecordBatch.from_arrays(
        [*batch.columns, output],
        schema=batch.schema.append(pyarrow.field(output_column.name, output.type)),
    )
    return batch_with_results


def _python_model_predict(model: PythonModelContainer, input_batch: pyarrow.RecordBatch) -> numpy.ndarray:
    return model.predict(input_batch)


def _convert_numpy_to_arrow(
    output: numpy.ndarray, num_of_rows: int, output_column_type: pyarrow.DataType
) -> pyarrow.Array:
    # If the output is scalar but is 2d (n x 1), then conversion to pyArrow will fail with scalar output_column_type
    # so we need to remove that extra dimension
    if output.ndim == 2 and not isinstance(output_column_type, pyarrow.ListType):
        output = output.squeeze()

    if output.shape[0] != num_of_rows:
        msg = "Output of model should have a single row per input row. "
        raise TectonValidationError(msg)
    return pyarrow.array(list(output), type=output_column_type)


@dataclass
class PythonModelInferenceFuncConfig:
    _model_dir: str
    _model_file_path: str
    _extra_kwargs: Mapping[str, Any]

    # Note: _final_kwargs is a late init object.
    _final_kwargs: Optional[Mapping[str, Any]] = None

    @classmethod
    def create(
        cls,
        model_dir: str,
        model_file_path: str,
        input_columns: List[schema.Column],
        output_column: schema.Column,
        model_input_schema: List[schema.Column],
    ) -> PythonModelInferenceFuncConfig:
        return cls(
            _model_dir=model_dir,
            _model_file_path=model_file_path,
            _extra_kwargs=MappingProxyType(
                {
                    "input_columns": input_columns,
                    "output_column": output_column,
                    "model_input_schema": model_input_schema,
                }
            ),
        )

    def load(self):
        model_container = PythonModelContainer(data_dir=self._model_dir, model_file_path=self._model_file_path)
        model_container.load()
        self._final_kwargs = MappingProxyType(dict(model=model_container, **self._extra_kwargs))
        return self

    def kwargs(self) -> Mapping[str, Any]:
        if self._final_kwargs is None:
            msg = "`load` must be called prior to calling `kwargs`."
            raise ValueError(msg)
        return self._final_kwargs



from tecton_proto.spark_common import clusters__client_pb2 as _clusters__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

AWS_INSUFFICIENT_INSTANCE_CAPACITY_FAILURE: TerminationCode
CLIENT_ERROR: TerminationType
CLOUD_FAILURE: TerminationType
CLOUD_PROVIDER_LAUNCH_FAILURE: TerminationCode
CLOUD_PROVIDER_SHUTDOWN: TerminationCode
COMMUNICATION_LOST: TerminationCode
CONTAINER_LAUNCH_FAILURE: TerminationCode
DBFS_COMPONENT_UNHEALTHY: TerminationCode
DESCRIPTOR: _descriptor.FileDescriptor
DRIVER_UNREACHABLE: TerminationCode
DRIVER_UNRESPONSIVE: TerminationCode
ERROR: ClusterState
INACTIVITY: TerminationCode
INIT_SCRIPT_FAILURE: TerminationCode
INSTANCE_POOL_CLUSTER_FAILURE: TerminationCode
INSTANCE_UNREACHABLE: TerminationCode
INTERNAL_ERROR: TerminationCode
INVALID_ARGUMENT: TerminationCode
JOB_FINISHED: TerminationCode
METASTORE_COMPONENT_UNHEALTHY: TerminationCode
PENDING: ClusterState
REQUEST_REJECTED: TerminationCode
RESIZING: ClusterState
RESTARTING: ClusterState
RUNNING: ClusterState
SERVICE_FAULT: TerminationType
SPARK_ERROR: TerminationCode
SPARK_STARTUP_FAILURE: TerminationCode
SUCCESS: TerminationType
TERMINATED: ClusterState
TERMINATING: ClusterState
TRIAL_EXPIRED: TerminationCode
UNEXPECTED_LAUNCH_FAILURE: TerminationCode
UNKNOWN: ClusterState
UNKNOWN_TERMINATION_STATE: TerminationCode
UNKNOWN_TERMINATION_TYPE: TerminationType
USER_REQUEST: TerminationCode

class Cluster(_message.Message):
    __slots__ = ["cluster_id", "cluster_name", "custom_tags", "spark_version", "state"]
    class CustomTagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    CLUSTER_ID_FIELD_NUMBER: _ClassVar[int]
    CLUSTER_NAME_FIELD_NUMBER: _ClassVar[int]
    CUSTOM_TAGS_FIELD_NUMBER: _ClassVar[int]
    SPARK_VERSION_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    cluster_id: str
    cluster_name: str
    custom_tags: _containers.ScalarMap[str, str]
    spark_version: str
    state: ClusterState
    def __init__(self, cluster_id: _Optional[str] = ..., cluster_name: _Optional[str] = ..., spark_version: _Optional[str] = ..., state: _Optional[_Union[ClusterState, str]] = ..., custom_tags: _Optional[_Mapping[str, str]] = ...) -> None: ...

class ClusterAutoScale(_message.Message):
    __slots__ = ["max_workers", "min_workers"]
    MAX_WORKERS_FIELD_NUMBER: _ClassVar[int]
    MIN_WORKERS_FIELD_NUMBER: _ClassVar[int]
    max_workers: int
    min_workers: int
    def __init__(self, min_workers: _Optional[int] = ..., max_workers: _Optional[int] = ...) -> None: ...

class ClusterCreateRequest(_message.Message):
    __slots__ = ["apply_policy_default_values", "autoscale", "autotermination_minutes", "aws_attributes", "cluster_name", "custom_tags", "data_security_mode", "driver_node_type_id", "enable_elastic_disk", "gcp_attributes", "idempotency_token", "init_scripts", "node_type_id", "num_workers", "policy_id", "single_user_name", "spark_conf", "spark_env_vars", "spark_version"]
    class CustomTagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    class SparkConfEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    class SparkEnvVarsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    APPLY_POLICY_DEFAULT_VALUES_FIELD_NUMBER: _ClassVar[int]
    AUTOSCALE_FIELD_NUMBER: _ClassVar[int]
    AUTOTERMINATION_MINUTES_FIELD_NUMBER: _ClassVar[int]
    AWS_ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    CLUSTER_NAME_FIELD_NUMBER: _ClassVar[int]
    CUSTOM_TAGS_FIELD_NUMBER: _ClassVar[int]
    DATA_SECURITY_MODE_FIELD_NUMBER: _ClassVar[int]
    DRIVER_NODE_TYPE_ID_FIELD_NUMBER: _ClassVar[int]
    ENABLE_ELASTIC_DISK_FIELD_NUMBER: _ClassVar[int]
    GCP_ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_TOKEN_FIELD_NUMBER: _ClassVar[int]
    INIT_SCRIPTS_FIELD_NUMBER: _ClassVar[int]
    NODE_TYPE_ID_FIELD_NUMBER: _ClassVar[int]
    NUM_WORKERS_FIELD_NUMBER: _ClassVar[int]
    POLICY_ID_FIELD_NUMBER: _ClassVar[int]
    SINGLE_USER_NAME_FIELD_NUMBER: _ClassVar[int]
    SPARK_CONF_FIELD_NUMBER: _ClassVar[int]
    SPARK_ENV_VARS_FIELD_NUMBER: _ClassVar[int]
    SPARK_VERSION_FIELD_NUMBER: _ClassVar[int]
    apply_policy_default_values: bool
    autoscale: ClusterAutoScale
    autotermination_minutes: int
    aws_attributes: _clusters__client_pb2.AwsAttributes
    cluster_name: str
    custom_tags: _containers.ScalarMap[str, str]
    data_security_mode: str
    driver_node_type_id: str
    enable_elastic_disk: bool
    gcp_attributes: _clusters__client_pb2.GCPAttributes
    idempotency_token: str
    init_scripts: _containers.RepeatedCompositeFieldContainer[_clusters__client_pb2.ResourceLocation]
    node_type_id: str
    num_workers: int
    policy_id: str
    single_user_name: str
    spark_conf: _containers.ScalarMap[str, str]
    spark_env_vars: _containers.ScalarMap[str, str]
    spark_version: str
    def __init__(self, spark_conf: _Optional[_Mapping[str, str]] = ..., driver_node_type_id: _Optional[str] = ..., node_type_id: _Optional[str] = ..., num_workers: _Optional[int] = ..., cluster_name: _Optional[str] = ..., spark_version: _Optional[str] = ..., aws_attributes: _Optional[_Union[_clusters__client_pb2.AwsAttributes, _Mapping]] = ..., idempotency_token: _Optional[str] = ..., spark_env_vars: _Optional[_Mapping[str, str]] = ..., custom_tags: _Optional[_Mapping[str, str]] = ..., autotermination_minutes: _Optional[int] = ..., enable_elastic_disk: bool = ..., autoscale: _Optional[_Union[ClusterAutoScale, _Mapping]] = ..., init_scripts: _Optional[_Iterable[_Union[_clusters__client_pb2.ResourceLocation, _Mapping]]] = ..., policy_id: _Optional[str] = ..., gcp_attributes: _Optional[_Union[_clusters__client_pb2.GCPAttributes, _Mapping]] = ..., data_security_mode: _Optional[str] = ..., single_user_name: _Optional[str] = ..., apply_policy_default_values: bool = ...) -> None: ...

class ClusterCreateResponse(_message.Message):
    __slots__ = ["cluster_id"]
    CLUSTER_ID_FIELD_NUMBER: _ClassVar[int]
    cluster_id: str
    def __init__(self, cluster_id: _Optional[str] = ...) -> None: ...

class ClusterListResponse(_message.Message):
    __slots__ = ["clusters"]
    CLUSTERS_FIELD_NUMBER: _ClassVar[int]
    clusters: _containers.RepeatedCompositeFieldContainer[Cluster]
    def __init__(self, clusters: _Optional[_Iterable[_Union[Cluster, _Mapping]]] = ...) -> None: ...

class ClusterTerminateRequest(_message.Message):
    __slots__ = ["cluster_id"]
    CLUSTER_ID_FIELD_NUMBER: _ClassVar[int]
    cluster_id: str
    def __init__(self, cluster_id: _Optional[str] = ...) -> None: ...

class ClustersGetRequest(_message.Message):
    __slots__ = ["cluster_id"]
    CLUSTER_ID_FIELD_NUMBER: _ClassVar[int]
    cluster_id: str
    def __init__(self, cluster_id: _Optional[str] = ...) -> None: ...

class ClustersGetResponse(_message.Message):
    __slots__ = ["cluster_id", "state", "state_message", "termination_reason"]
    CLUSTER_ID_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    STATE_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    TERMINATION_REASON_FIELD_NUMBER: _ClassVar[int]
    cluster_id: str
    state: ClusterState
    state_message: str
    termination_reason: TerminationReason
    def __init__(self, cluster_id: _Optional[str] = ..., state_message: _Optional[str] = ..., termination_reason: _Optional[_Union[TerminationReason, _Mapping]] = ..., state: _Optional[_Union[ClusterState, str]] = ...) -> None: ...

class TerminationReason(_message.Message):
    __slots__ = ["code", "type"]
    CODE_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    code: TerminationCode
    type: TerminationType
    def __init__(self, code: _Optional[_Union[TerminationCode, str]] = ..., type: _Optional[_Union[TerminationType, str]] = ...) -> None: ...

class ClusterState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class TerminationCode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class TerminationType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
Tecton's comment:
This is a backport from pyspark 3.5.1
We need it because of the fixed spark -> arrow schema conversion.
pyspark 3.1 doesn't support TimestampType nested into Map or Struct
"""

from typing import TYPE_CHECKING

from pyspark.sql.types import (
    BooleanType,
    ByteType,
    ShortType,
    IntegerType,
    IntegralType,
    LongType,
    FloatType,
    DoubleType,
    DecimalType,
    StringType,
    BinaryType,
    DateType,
    TimestampType,
    ArrayType,
    MapType,
    StructType,
    StructField,
    NullType,
    DataType,
    UserDefinedType,
    Row,
    _create_row,
)

if TYPE_CHECKING:
    import pyarrow as pa


def to_arrow_type(dt: DataType) -> "pa.DataType":
    """Convert Spark data type to pyarrow type"""
    from distutils.version import LooseVersion
    import pyarrow as pa

    if type(dt) == BooleanType:
        arrow_type = pa.bool_()
    elif type(dt) == ByteType:
        arrow_type = pa.int8()
    elif type(dt) == ShortType:
        arrow_type = pa.int16()
    elif type(dt) == IntegerType:
        arrow_type = pa.int32()
    elif type(dt) == LongType:
        arrow_type = pa.int64()
    elif type(dt) == FloatType:
        arrow_type = pa.float32()
    elif type(dt) == DoubleType:
        arrow_type = pa.float64()
    elif type(dt) == DecimalType:
        arrow_type = pa.decimal128(dt.precision, dt.scale)
    elif type(dt) == StringType:
        arrow_type = pa.string()
    elif type(dt) == BinaryType:
        arrow_type = pa.binary()
    elif type(dt) == DateType:
        arrow_type = pa.date32()
    # TODO: use pyspark type instead of name when we upgrade to Spark 3.4+
    elif type(dt) == TimestampType or dt.simpleString() == 'timestamp_ntz':
        # Timestamps should be in UTC, JVM Arrow timestamps require a timezone to be read
        arrow_type = pa.timestamp("us", tz="UTC")
    elif type(dt) == ArrayType:
        field = pa.field("element", to_arrow_type(dt.elementType), nullable=dt.containsNull)
        arrow_type = pa.list_(field)
    elif type(dt) == MapType:
        key_field = pa.field("key", to_arrow_type(dt.keyType), nullable=False)
        value_field = pa.field("value", to_arrow_type(dt.valueType), nullable=dt.valueContainsNull)
        arrow_type = pa.map_(key_field, value_field)
    elif type(dt) == StructType:
        fields = [
            pa.field(field.name, to_arrow_type(field.dataType), nullable=field.nullable)
            for field in dt
        ]
        arrow_type = pa.struct(fields)
    elif type(dt) == NullType:
        arrow_type = pa.null()
    elif isinstance(dt, UserDefinedType):
        arrow_type = to_arrow_type(dt.sqlType())
    else:
        raise TypeError("Unsupported type in conversion to Arrow: " + str(dt))
    return arrow_type


def to_arrow_schema(schema: StructType) -> "pa.Schema":
    """Convert a schema from Spark to Arrow"""
    import pyarrow as pa

    fields = [
        pa.field(field.name, to_arrow_type(field.dataType), nullable=field.nullable)
        for field in schema
    ]
    return pa.schema(fields)


def from_arrow_type(at: "pa.DataType", prefer_timestamp_ntz: bool = False) -> DataType:
    """Convert pyarrow type to Spark data type."""
    from distutils.version import LooseVersion
    import pyarrow as pa
    import pyarrow.types as types

    spark_type: DataType
    if types.is_boolean(at):
        spark_type = BooleanType()
    elif types.is_int8(at):
        spark_type = ByteType()
    elif types.is_int16(at):
        spark_type = ShortType()
    elif types.is_int32(at):
        spark_type = IntegerType()
    elif types.is_int64(at):
        spark_type = LongType()
    elif types.is_float32(at):
        spark_type = FloatType()
    elif types.is_float64(at):
        spark_type = DoubleType()
    elif types.is_decimal(at):
        spark_type = DecimalType(precision=at.precision, scale=at.scale)
    elif types.is_string(at):
        spark_type = StringType()
    elif types.is_large_string(at):
        spark_type = StringType()
    elif types.is_binary(at):
        spark_type = BinaryType()
    elif types.is_large_binary(at):
        spark_type = BinaryType()
    elif types.is_date32(at):
        spark_type = DateType()
    elif types.is_timestamp(at):
        spark_type = TimestampType()
    elif types.is_list(at):
        spark_type = ArrayType(from_arrow_type(at.value_type, prefer_timestamp_ntz))
    elif types.is_map(at):
        spark_type = MapType(
            from_arrow_type(at.key_type, prefer_timestamp_ntz),
            from_arrow_type(at.item_type, prefer_timestamp_ntz),
        )
    elif types.is_struct(at):
        return StructType(
            [
                StructField(
                    field.name,
                    from_arrow_type(field.type, prefer_timestamp_ntz),
                    nullable=field.nullable,
                )
                for field in at
            ]
        )
    elif types.is_dictionary(at):
        spark_type = from_arrow_type(at.value_type, prefer_timestamp_ntz)
    elif types.is_null(at):
        spark_type = NullType()
    else:
        raise TypeError("Unsupported type in conversion from Arrow: " + str(at))
    return spark_type


def from_arrow_schema(arrow_schema: "pa.Schema", prefer_timestamp_ntz: bool = False) -> StructType:
    """Convert schema from Arrow to Spark."""
    return StructType(
        [
            StructField(
                field.name,
                from_arrow_type(field.type, prefer_timestamp_ntz),
                nullable=field.nullable,
            )
            for field in arrow_schema
        ]
    )



import logging
import tempfile
import time
from dataclasses import dataclass
from typing import TYPE_CHECKING
from typing import Optional

from tecton_core import _gen_version
from tecton_core import conf


if TYPE_CHECKING:
    from duckdb import DuckDBPyConnection

"""
Factory code for creating DuckDB connections.
"""

logger = logging.getLogger(__name__)


def get_ext_version():
    return "latest" if _gen_version.VERSION == "99.99.99" else _gen_version.VERSION


_home_dir_override: Optional[str] = None


def set_home_dir_override(value: Optional[str]) -> None:
    global _home_dir_override
    _home_dir_override = value


@dataclass
# Example: DuckDBConfig(memory_limit='1GB', num_threads=4)
class DuckDBConfig:
    memory_limit_in_bytes: Optional[int] = None  # Memory limit for DuckDB
    num_threads: Optional[int] = None  # Number of threads for DuckDB
    use_unique_extension_path: bool = False  # Use unique extension path for each connection, this is needed when we can multiple Ray tasks on the same machine and want to separate them.


def run_duckdb_sql_with_retry(connection, sql_str, max_retries=3, wait_seconds=2):
    try:
        import duckdb
    except ImportError:
        msg = (
            "Couldn't initialize DuckDB connection. "
            "Please install DuckDB dependencies by executing `pip install tecton[rift]`."
        )
        raise RuntimeError(msg)

    retries = 0
    while retries < max_retries:
        try:
            connection.sql(sql_str)
            return
        except duckdb.IOException as e:
            retries += 1
            if retries < max_retries:
                time.sleep(wait_seconds)
            else:
                logger.fatal(f"Duckdb Sql {sql_str} Failed after {max_retries} retries.")
                raise


def create_connection(duckdb_config: Optional[DuckDBConfig] = None) -> "DuckDBPyConnection":
    """
    Create a new instance of DuckDBPyConnection.
    """
    try:
        import duckdb
    except ImportError:
        msg = (
            "Couldn't initialize DuckDB connection. "
            "Please install DuckDB dependencies by executing `pip install tecton[rift]`."
        )
        raise RuntimeError(msg)

    conn_config = {}
    if conf.get_or_none("DUCKDB_EXTENSION_REPO"):
        conn_config["allow_unsigned_extensions"] = "true"

    if conf.get_bool("DUCKDB_PERSIST_DB"):
        connection = duckdb.connect("duckdb.db", config=conn_config)
    else:
        connection = duckdb.connect(config=conn_config)

    # Initialize the DuckDB connection
    if _home_dir_override:
        connection.sql(f"SET home_directory='{_home_dir_override}'")

    # TODO(liangqi): Remove this once we move to packaging extensions into Python package.
    if duckdb_config and duckdb_config.use_unique_extension_path:
        # Use mkdtemp instead of TemporaryDirectory to keep the directory alive after the function. We cannot set
        # `delete=False` for TemporaryDirectory until Python 3.12
        temporary_extension_directory = tempfile.mkdtemp(suffix="duckdb_ext_directory_")
        connection.sql(f"SET extension_directory = '{temporary_extension_directory}'")

    connection.sql("INSTALL httpfs;")
    run_duckdb_sql_with_retry(connection, "LOAD httpfs;")
    connection.sql(f"SET http_retries='{conf.get_or_raise('DUCKDB_HTTP_RETRIES')}'")

    if conf.get_bool("DUCKDB_DISK_SPILLING_ENABLED"):
        # The directory will be deleted when the TemporaryDirectory object is destroyed even if we don't call
        # __enter__. This means as long as we store the object somewhere the directory will live as the context and
        # will be cleaned up at interpreter exit.
        temporary_directory = tempfile.TemporaryDirectory(suffix=".tecton_duckdb")
        connection.sql(f"SET temp_directory = '{temporary_directory.name}'")

    duckdb_memory_limit = (
        f"{duckdb_config.memory_limit_in_bytes // 1024 // 1024}MB"
        if duckdb_config and duckdb_config.memory_limit_in_bytes
        else conf.get_or_none("DUCKDB_MEMORY_LIMIT")
    )
    if duckdb_memory_limit:
        if conf.get_bool("DUCKDB_DEBUG"):
            print(f"Setting duckdb memory limit to {duckdb_memory_limit}")

        connection.sql(f"SET memory_limit='{duckdb_memory_limit}'")

    num_duckdb_threads = (
        duckdb_config.num_threads
        if duckdb_config and duckdb_config.num_threads
        else conf.get_or_none("DUCKDB_NTHREADS")
    )
    if num_duckdb_threads:
        connection.sql(f"SET threads TO {num_duckdb_threads};")
        if conf.get_bool("DUCKDB_DEBUG"):
            print(f"Setting duckdb threads to {num_duckdb_threads}")

    # Workaround for pypika not supporting the // operator
    connection.sql("CREATE OR REPLACE MACRO _tecton_int_div(a, b) AS a // b")
    extension_repo = conf.get_or_none("DUCKDB_EXTENSION_REPO")
    if extension_repo:
        versioned_extension_repo = extension_repo.format(version=get_ext_version())
        connection.sql(f"SET custom_extension_repository='{versioned_extension_repo}'")
        if conf.get_bool("DUCKDB_ALLOW_CACHE_EXTENSION"):
            # Allow using local cached version of extension
            connection.sql("INSTALL tecton")
        else:
            # Always download the latest version of the duckdb extension
            connection.sql("FORCE INSTALL tecton")
        run_duckdb_sql_with_retry(connection, "LOAD tecton")

    connection.sql("SET TimeZone='UTC'")

    return connection



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

AGGREGATION_FUNCTION_APPROX_COUNT_DISTINCT: AggregationFunction
AGGREGATION_FUNCTION_APPROX_PERCENTILE: AggregationFunction
AGGREGATION_FUNCTION_COUNT: AggregationFunction
AGGREGATION_FUNCTION_FIRST_DISTINCT_N: AggregationFunction
AGGREGATION_FUNCTION_FIRST_NON_DISTINCT_N: AggregationFunction
AGGREGATION_FUNCTION_LAST: AggregationFunction
AGGREGATION_FUNCTION_LAST_DISTINCT_N: AggregationFunction
AGGREGATION_FUNCTION_LAST_NON_DISTINCT_N: AggregationFunction
AGGREGATION_FUNCTION_MAX: AggregationFunction
AGGREGATION_FUNCTION_MEAN: AggregationFunction
AGGREGATION_FUNCTION_MIN: AggregationFunction
AGGREGATION_FUNCTION_STDDEV_POP: AggregationFunction
AGGREGATION_FUNCTION_STDDEV_SAMP: AggregationFunction
AGGREGATION_FUNCTION_SUM: AggregationFunction
AGGREGATION_FUNCTION_UNKNOWN: AggregationFunction
AGGREGATION_FUNCTION_VAR_POP: AggregationFunction
AGGREGATION_FUNCTION_VAR_SAMP: AggregationFunction
DESCRIPTOR: _descriptor.FileDescriptor

class AggregationFunctionParams(_message.Message):
    __slots__ = ["approx_count_distinct", "approx_percentile", "first_n", "last_n"]
    APPROX_COUNT_DISTINCT_FIELD_NUMBER: _ClassVar[int]
    APPROX_PERCENTILE_FIELD_NUMBER: _ClassVar[int]
    FIRST_N_FIELD_NUMBER: _ClassVar[int]
    LAST_N_FIELD_NUMBER: _ClassVar[int]
    approx_count_distinct: ApproxCountDistinctParams
    approx_percentile: ApproxPercentileParams
    first_n: FirstNParams
    last_n: LastNParams
    def __init__(self, last_n: _Optional[_Union[LastNParams, _Mapping]] = ..., first_n: _Optional[_Union[FirstNParams, _Mapping]] = ..., approx_count_distinct: _Optional[_Union[ApproxCountDistinctParams, _Mapping]] = ..., approx_percentile: _Optional[_Union[ApproxPercentileParams, _Mapping]] = ...) -> None: ...

class ApproxCountDistinctParams(_message.Message):
    __slots__ = ["precision"]
    PRECISION_FIELD_NUMBER: _ClassVar[int]
    precision: int
    def __init__(self, precision: _Optional[int] = ...) -> None: ...

class ApproxPercentileParams(_message.Message):
    __slots__ = ["percentile", "precision"]
    PERCENTILE_FIELD_NUMBER: _ClassVar[int]
    PRECISION_FIELD_NUMBER: _ClassVar[int]
    percentile: float
    precision: int
    def __init__(self, percentile: _Optional[float] = ..., precision: _Optional[int] = ...) -> None: ...

class FirstNParams(_message.Message):
    __slots__ = ["n"]
    N_FIELD_NUMBER: _ClassVar[int]
    n: int
    def __init__(self, n: _Optional[int] = ...) -> None: ...

class LastNParams(_message.Message):
    __slots__ = ["n"]
    N_FIELD_NUMBER: _ClassVar[int]
    n: int
    def __init__(self, n: _Optional[int] = ...) -> None: ...

class AggregationFunction(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from typing import Any
from typing import Mapping
from typing import Optional


class MockContext:
    """
    MockContext is a class that is used to pass mock context metadata such as secrets and resources for
    testing.
    """

    _secrets: Optional[Mapping[str, str]] = None
    _resources: Optional[Mapping[str, Any]] = None

    def __init__(
        self,
        secrets: Optional[Mapping[str, str]] = None,
        resources: Optional[Mapping[str, Any]] = None,
    ) -> None:
        """
        Initialize the MockContext object.

        Example:
        ```python
        MockContext(
            secrets={"my_api_key": "1234567890"},
            resources={"my_resource": {"client_version": "1.0.0"}}
        )
        ```

        :param secrets: A map that maps the secret name to the resolved secret value.
        :param resources: A map that maps the resource name to the resource returned by the resource provider.
        """
        self._secrets = secrets
        self._resources = resources

    def __str__(self) -> str:
        return f"MockContext(secrets={self._secrets}, " f"resources={self._resources})"

    @property
    def secrets(self) -> Optional[Mapping[str, str]]:
        return self._secrets

    @property
    def resources(self) -> Optional[Mapping[str, Any]]:
        return self._resources



from tecton_proto.data import fco_metadata__client_pb2 as _fco_metadata__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class FcoSummary(_message.Message):
    __slots__ = ["fco_metadata", "summary_items"]
    FCO_METADATA_FIELD_NUMBER: _ClassVar[int]
    SUMMARY_ITEMS_FIELD_NUMBER: _ClassVar[int]
    fco_metadata: _fco_metadata__client_pb2.FcoMetadata
    summary_items: _containers.RepeatedCompositeFieldContainer[SummaryItem]
    def __init__(self, fco_metadata: _Optional[_Union[_fco_metadata__client_pb2.FcoMetadata, _Mapping]] = ..., summary_items: _Optional[_Iterable[_Union[SummaryItem, _Mapping]]] = ...) -> None: ...

class SummaryItem(_message.Message):
    __slots__ = ["display_name", "key", "multi_values", "nested_summary_items", "value"]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    KEY_FIELD_NUMBER: _ClassVar[int]
    MULTI_VALUES_FIELD_NUMBER: _ClassVar[int]
    NESTED_SUMMARY_ITEMS_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    display_name: str
    key: str
    multi_values: _containers.RepeatedScalarFieldContainer[str]
    nested_summary_items: _containers.RepeatedCompositeFieldContainer[SummaryItem]
    value: str
    def __init__(self, key: _Optional[str] = ..., display_name: _Optional[str] = ..., value: _Optional[str] = ..., multi_values: _Optional[_Iterable[str]] = ..., nested_summary_items: _Optional[_Iterable[_Union[SummaryItem, _Mapping]]] = ...) -> None: ...



from tecton_proto.args import basic_info__client_pb2 as _basic_info__client_pb2
from tecton_proto.args import diff_options__client_pb2 as _diff_options__client_pb2
from tecton_proto.common import framework_version__client_pb2 as _framework_version__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.common import schema__client_pb2 as _schema__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class EntityArgs(_message.Message):
    __slots__ = ["entity_id", "info", "join_keys", "join_keys_legacy", "options", "prevent_destroy", "version"]
    class OptionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ENTITY_ID_FIELD_NUMBER: _ClassVar[int]
    INFO_FIELD_NUMBER: _ClassVar[int]
    JOIN_KEYS_FIELD_NUMBER: _ClassVar[int]
    JOIN_KEYS_LEGACY_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    PREVENT_DESTROY_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    entity_id: _id__client_pb2.Id
    info: _basic_info__client_pb2.BasicInfo
    join_keys: _containers.RepeatedCompositeFieldContainer[_schema__client_pb2.Column]
    join_keys_legacy: _containers.RepeatedScalarFieldContainer[str]
    options: _containers.ScalarMap[str, str]
    prevent_destroy: bool
    version: _framework_version__client_pb2.FrameworkVersion
    def __init__(self, entity_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., info: _Optional[_Union[_basic_info__client_pb2.BasicInfo, _Mapping]] = ..., version: _Optional[_Union[_framework_version__client_pb2.FrameworkVersion, str]] = ..., prevent_destroy: bool = ..., options: _Optional[_Mapping[str, str]] = ..., join_keys_legacy: _Optional[_Iterable[str]] = ..., join_keys: _Optional[_Iterable[_Union[_schema__client_pb2.Column, _Mapping]]] = ...) -> None: ...



from tecton_proto.auditlog import metadata__client_pb2 as _metadata__client_pb2
from tecton_proto.auth import service__client_pb2 as _service__client_pb2
from tecton_proto.data import service_account__client_pb2 as _service_account__client_pb2
from tecton_proto.metadataservice import metadata_service__client_pb2 as _metadata_service__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ActivateServiceAccountSecretRequest(_message.Message):
    __slots__ = ["secret_id", "service_account_id"]
    SECRET_ID_FIELD_NUMBER: _ClassVar[int]
    SERVICE_ACCOUNT_ID_FIELD_NUMBER: _ClassVar[int]
    secret_id: str
    service_account_id: str
    def __init__(self, service_account_id: _Optional[str] = ..., secret_id: _Optional[str] = ...) -> None: ...

class ActivateServiceAccountSecretResponse(_message.Message):
    __slots__ = ["secret"]
    SECRET_FIELD_NUMBER: _ClassVar[int]
    secret: _service_account__client_pb2.MaskedClientSecret
    def __init__(self, secret: _Optional[_Union[_service_account__client_pb2.MaskedClientSecret, _Mapping]] = ...) -> None: ...

class CreateServiceAccountSecretRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class CreateServiceAccountSecretResponse(_message.Message):
    __slots__ = ["secret"]
    SECRET_FIELD_NUMBER: _ClassVar[int]
    secret: _service_account__client_pb2.NewClientSecret
    def __init__(self, secret: _Optional[_Union[_service_account__client_pb2.NewClientSecret, _Mapping]] = ...) -> None: ...

class DeactivateServiceAccountSecretRequest(_message.Message):
    __slots__ = ["secret_id", "service_account_id"]
    SECRET_ID_FIELD_NUMBER: _ClassVar[int]
    SERVICE_ACCOUNT_ID_FIELD_NUMBER: _ClassVar[int]
    secret_id: str
    service_account_id: str
    def __init__(self, service_account_id: _Optional[str] = ..., secret_id: _Optional[str] = ...) -> None: ...

class DeactivateServiceAccountSecretResponse(_message.Message):
    __slots__ = ["secret"]
    SECRET_FIELD_NUMBER: _ClassVar[int]
    secret: _service_account__client_pb2.MaskedClientSecret
    def __init__(self, secret: _Optional[_Union[_service_account__client_pb2.MaskedClientSecret, _Mapping]] = ...) -> None: ...

class DeleteServiceAccountSecretRequest(_message.Message):
    __slots__ = ["secret_id", "service_account_id"]
    SECRET_ID_FIELD_NUMBER: _ClassVar[int]
    SERVICE_ACCOUNT_ID_FIELD_NUMBER: _ClassVar[int]
    secret_id: str
    service_account_id: str
    def __init__(self, service_account_id: _Optional[str] = ..., secret_id: _Optional[str] = ...) -> None: ...

class DeleteServiceAccountSecretResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ListServiceAccountSecretsRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class ListServiceAccountSecretsResponse(_message.Message):
    __slots__ = ["client_secrets"]
    CLIENT_SECRETS_FIELD_NUMBER: _ClassVar[int]
    client_secrets: _containers.RepeatedCompositeFieldContainer[_service_account__client_pb2.MaskedClientSecret]
    def __init__(self, client_secrets: _Optional[_Iterable[_Union[_service_account__client_pb2.MaskedClientSecret, _Mapping]]] = ...) -> None: ...



from tecton_proto.auditlog import metadata__client_pb2 as _metadata__client_pb2
from tecton_proto.auth import principal__client_pb2 as _principal__client_pb2
from tecton_proto.auth import resource__client_pb2 as _resource__client_pb2
from tecton_proto.auth import resource_role_assignments__client_pb2 as _resource_role_assignments__client_pb2
from tecton_proto.auth import service__client_pb2 as _service__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AssignRolesPutRequest(_message.Message):
    __slots__ = ["principal_id", "principal_type", "resource_id", "resource_type", "roles"]
    PRINCIPAL_ID_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_TYPE_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_ID_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    principal_id: str
    principal_type: _principal__client_pb2.PrincipalType
    resource_id: str
    resource_type: _resource__client_pb2.ResourceType
    roles: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, resource_type: _Optional[_Union[_resource__client_pb2.ResourceType, str]] = ..., resource_id: _Optional[str] = ..., principal_type: _Optional[_Union[_principal__client_pb2.PrincipalType, str]] = ..., principal_id: _Optional[str] = ..., roles: _Optional[_Iterable[str]] = ...) -> None: ...

class AssignRolesPutResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class AssignRolesRequest(_message.Message):
    __slots__ = ["assignments"]
    ASSIGNMENTS_FIELD_NUMBER: _ClassVar[int]
    assignments: _containers.RepeatedCompositeFieldContainer[Assignment]
    def __init__(self, assignments: _Optional[_Iterable[_Union[Assignment, _Mapping]]] = ...) -> None: ...

class AssignRolesResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class Assignment(_message.Message):
    __slots__ = ["principal_id", "principal_type", "resource_id", "resource_type", "role"]
    PRINCIPAL_ID_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_TYPE_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_ID_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    ROLE_FIELD_NUMBER: _ClassVar[int]
    principal_id: str
    principal_type: _principal__client_pb2.PrincipalType
    resource_id: str
    resource_type: _resource__client_pb2.ResourceType
    role: str
    def __init__(self, resource_type: _Optional[_Union[_resource__client_pb2.ResourceType, str]] = ..., resource_id: _Optional[str] = ..., role: _Optional[str] = ..., principal_type: _Optional[_Union[_principal__client_pb2.PrincipalType, str]] = ..., principal_id: _Optional[str] = ...) -> None: ...

class AssignmentBasic(_message.Message):
    __slots__ = ["principal", "role_assignments", "roles"]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    ROLE_ASSIGNMENTS_FIELD_NUMBER: _ClassVar[int]
    principal: _principal__client_pb2.PrincipalBasic
    role_assignments: _containers.RepeatedCompositeFieldContainer[_resource_role_assignments__client_pb2.ResourceAndRoleAssignments]
    roles: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, principal: _Optional[_Union[_principal__client_pb2.PrincipalBasic, _Mapping]] = ..., roles: _Optional[_Iterable[str]] = ..., role_assignments: _Optional[_Iterable[_Union[_resource_role_assignments__client_pb2.ResourceAndRoleAssignments, _Mapping]]] = ...) -> None: ...

class AssignmentBasicV2(_message.Message):
    __slots__ = ["principal", "role_assignments"]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    ROLE_ASSIGNMENTS_FIELD_NUMBER: _ClassVar[int]
    principal: _principal__client_pb2.PrincipalBasic
    role_assignments: _containers.RepeatedCompositeFieldContainer[_resource_role_assignments__client_pb2.ResourceAndRoleAssignmentsV2]
    def __init__(self, principal: _Optional[_Union[_principal__client_pb2.PrincipalBasic, _Mapping]] = ..., role_assignments: _Optional[_Iterable[_Union[_resource_role_assignments__client_pb2.ResourceAndRoleAssignmentsV2, _Mapping]]] = ...) -> None: ...

class AuthorizedResources(_message.Message):
    __slots__ = ["actions", "resource_id"]
    ACTIONS_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_ID_FIELD_NUMBER: _ClassVar[int]
    actions: _containers.RepeatedScalarFieldContainer[str]
    resource_id: str
    def __init__(self, resource_id: _Optional[str] = ..., actions: _Optional[_Iterable[str]] = ...) -> None: ...

class GetAppPermissionsRequest(_message.Message):
    __slots__ = ["principal_id", "principal_type", "resource_type_permissions"]
    PRINCIPAL_ID_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_TYPE_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    principal_id: str
    principal_type: _principal__client_pb2.PrincipalType
    resource_type_permissions: _containers.RepeatedCompositeFieldContainer[ResourceTypePermissions]
    def __init__(self, principal_type: _Optional[_Union[_principal__client_pb2.PrincipalType, str]] = ..., principal_id: _Optional[str] = ..., resource_type_permissions: _Optional[_Iterable[_Union[ResourceTypePermissions, _Mapping]]] = ...) -> None: ...

class GetAppPermissionsResponse(_message.Message):
    __slots__ = ["resource_type_permission_values"]
    RESOURCE_TYPE_PERMISSION_VALUES_FIELD_NUMBER: _ClassVar[int]
    resource_type_permission_values: _containers.RepeatedCompositeFieldContainer[ResourceTypePermissionValues]
    def __init__(self, resource_type_permission_values: _Optional[_Iterable[_Union[ResourceTypePermissionValues, _Mapping]]] = ...) -> None: ...

class GetAssignedPrincipalsRequest(_message.Message):
    __slots__ = ["principal_types", "resource_id", "resource_type", "roles"]
    PRINCIPAL_TYPES_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_ID_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    principal_types: _containers.RepeatedScalarFieldContainer[_principal__client_pb2.PrincipalType]
    resource_id: str
    resource_type: _resource__client_pb2.ResourceType
    roles: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, resource_type: _Optional[_Union[_resource__client_pb2.ResourceType, str]] = ..., resource_id: _Optional[str] = ..., roles: _Optional[_Iterable[str]] = ..., principal_types: _Optional[_Iterable[_Union[_principal__client_pb2.PrincipalType, str]]] = ...) -> None: ...

class GetAssignedPrincipalsResponse(_message.Message):
    __slots__ = ["assignments"]
    ASSIGNMENTS_FIELD_NUMBER: _ClassVar[int]
    assignments: _containers.RepeatedCompositeFieldContainer[AssignmentBasic]
    def __init__(self, assignments: _Optional[_Iterable[_Union[AssignmentBasic, _Mapping]]] = ...) -> None: ...

class GetAssignedRolesRequest(_message.Message):
    __slots__ = ["principal_id", "principal_type", "resource_ids", "resource_type", "roles"]
    PRINCIPAL_ID_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_TYPE_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_IDS_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    principal_id: str
    principal_type: _principal__client_pb2.PrincipalType
    resource_ids: _containers.RepeatedScalarFieldContainer[str]
    resource_type: _resource__client_pb2.ResourceType
    roles: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, principal_type: _Optional[_Union[_principal__client_pb2.PrincipalType, str]] = ..., principal_id: _Optional[str] = ..., resource_type: _Optional[_Union[_resource__client_pb2.ResourceType, str]] = ..., resource_ids: _Optional[_Iterable[str]] = ..., roles: _Optional[_Iterable[str]] = ...) -> None: ...

class GetAssignedRolesResponse(_message.Message):
    __slots__ = ["assignments"]
    ASSIGNMENTS_FIELD_NUMBER: _ClassVar[int]
    assignments: _containers.RepeatedCompositeFieldContainer[_resource_role_assignments__client_pb2.ResourceAndRoleAssignments]
    def __init__(self, assignments: _Optional[_Iterable[_Union[_resource_role_assignments__client_pb2.ResourceAndRoleAssignments, _Mapping]]] = ...) -> None: ...

class GetAuthorizedResourcesRequest(_message.Message):
    __slots__ = ["action", "principal_id", "principal_type", "resource_type"]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_ID_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_TYPE_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    action: str
    principal_id: str
    principal_type: _principal__client_pb2.PrincipalType
    resource_type: _resource__client_pb2.ResourceType
    def __init__(self, principal_type: _Optional[_Union[_principal__client_pb2.PrincipalType, str]] = ..., principal_id: _Optional[str] = ..., resource_type: _Optional[_Union[_resource__client_pb2.ResourceType, str]] = ..., action: _Optional[str] = ...) -> None: ...

class GetAuthorizedResourcesResponse(_message.Message):
    __slots__ = ["authorized_resources"]
    AUTHORIZED_RESOURCES_FIELD_NUMBER: _ClassVar[int]
    authorized_resources: _containers.RepeatedCompositeFieldContainer[AuthorizedResources]
    def __init__(self, authorized_resources: _Optional[_Iterable[_Union[AuthorizedResources, _Mapping]]] = ...) -> None: ...

class GetIsAuthorizedRequest(_message.Message):
    __slots__ = ["permissions", "principal_id", "principal_type"]
    PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_ID_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_TYPE_FIELD_NUMBER: _ClassVar[int]
    permissions: _containers.RepeatedCompositeFieldContainer[Permission]
    principal_id: str
    principal_type: _principal__client_pb2.PrincipalType
    def __init__(self, principal_type: _Optional[_Union[_principal__client_pb2.PrincipalType, str]] = ..., principal_id: _Optional[str] = ..., permissions: _Optional[_Iterable[_Union[Permission, _Mapping]]] = ...) -> None: ...

class GetIsAuthorizedResponse(_message.Message):
    __slots__ = ["permissions"]
    PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    permissions: _containers.RepeatedCompositeFieldContainer[Permission]
    def __init__(self, permissions: _Optional[_Iterable[_Union[Permission, _Mapping]]] = ...) -> None: ...

class GetRolesRequest(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class GetRolesResponse(_message.Message):
    __slots__ = ["roles"]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    roles: _containers.RepeatedCompositeFieldContainer[RoleDefinition]
    def __init__(self, roles: _Optional[_Iterable[_Union[RoleDefinition, _Mapping]]] = ...) -> None: ...

class GetWorkspacePermissionsRequest(_message.Message):
    __slots__ = ["principal_id", "principal_type", "resource_type_permissions", "workspace_id"]
    PRINCIPAL_ID_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_TYPE_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_ID_FIELD_NUMBER: _ClassVar[int]
    principal_id: str
    principal_type: _principal__client_pb2.PrincipalType
    resource_type_permissions: _containers.RepeatedCompositeFieldContainer[ResourceTypePermissions]
    workspace_id: str
    def __init__(self, principal_type: _Optional[_Union[_principal__client_pb2.PrincipalType, str]] = ..., principal_id: _Optional[str] = ..., workspace_id: _Optional[str] = ..., resource_type_permissions: _Optional[_Iterable[_Union[ResourceTypePermissions, _Mapping]]] = ...) -> None: ...

class GetWorkspacePermissionsResponse(_message.Message):
    __slots__ = ["resource_type_permission_values"]
    RESOURCE_TYPE_PERMISSION_VALUES_FIELD_NUMBER: _ClassVar[int]
    resource_type_permission_values: _containers.RepeatedCompositeFieldContainer[ResourceTypePermissionValues]
    def __init__(self, resource_type_permission_values: _Optional[_Iterable[_Union[ResourceTypePermissionValues, _Mapping]]] = ...) -> None: ...

class ListAssignedPrincipalsRequest(_message.Message):
    __slots__ = ["principal_types", "resource_id", "resource_type", "roles"]
    PRINCIPAL_TYPES_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_ID_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    principal_types: _containers.RepeatedScalarFieldContainer[_principal__client_pb2.PrincipalType]
    resource_id: str
    resource_type: _resource__client_pb2.ResourceType
    roles: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, resource_type: _Optional[_Union[_resource__client_pb2.ResourceType, str]] = ..., resource_id: _Optional[str] = ..., roles: _Optional[_Iterable[str]] = ..., principal_types: _Optional[_Iterable[_Union[_principal__client_pb2.PrincipalType, str]]] = ...) -> None: ...

class ListAssignedPrincipalsResponse(_message.Message):
    __slots__ = ["assignments"]
    ASSIGNMENTS_FIELD_NUMBER: _ClassVar[int]
    assignments: _containers.RepeatedCompositeFieldContainer[AssignmentBasicV2]
    def __init__(self, assignments: _Optional[_Iterable[_Union[AssignmentBasicV2, _Mapping]]] = ...) -> None: ...

class ListAssignedRolesRequest(_message.Message):
    __slots__ = ["principal_id", "principal_type", "resource_ids", "resource_type", "roles"]
    PRINCIPAL_ID_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_TYPE_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_IDS_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    principal_id: str
    principal_type: _principal__client_pb2.PrincipalType
    resource_ids: _containers.RepeatedScalarFieldContainer[str]
    resource_type: _resource__client_pb2.ResourceType
    roles: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, principal_type: _Optional[_Union[_principal__client_pb2.PrincipalType, str]] = ..., principal_id: _Optional[str] = ..., resource_type: _Optional[_Union[_resource__client_pb2.ResourceType, str]] = ..., resource_ids: _Optional[_Iterable[str]] = ..., roles: _Optional[_Iterable[str]] = ...) -> None: ...

class ListAssignedRolesResponse(_message.Message):
    __slots__ = ["assignments"]
    ASSIGNMENTS_FIELD_NUMBER: _ClassVar[int]
    assignments: _containers.RepeatedCompositeFieldContainer[_resource_role_assignments__client_pb2.ResourceAndRoleAssignmentsV2]
    def __init__(self, assignments: _Optional[_Iterable[_Union[_resource_role_assignments__client_pb2.ResourceAndRoleAssignmentsV2, _Mapping]]] = ...) -> None: ...

class Permission(_message.Message):
    __slots__ = ["action", "resource_id", "resource_type"]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_ID_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    action: str
    resource_id: str
    resource_type: _resource__client_pb2.ResourceType
    def __init__(self, resource_type: _Optional[_Union[_resource__client_pb2.ResourceType, str]] = ..., resource_id: _Optional[str] = ..., action: _Optional[str] = ...) -> None: ...

class PermissionDefinition(_message.Message):
    __slots__ = ["description", "id", "is_authorized"]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    IS_AUTHORIZED_FIELD_NUMBER: _ClassVar[int]
    description: str
    id: str
    is_authorized: bool
    def __init__(self, id: _Optional[str] = ..., description: _Optional[str] = ..., is_authorized: bool = ...) -> None: ...

class PermissionValue(_message.Message):
    __slots__ = ["action", "is_authorized"]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    IS_AUTHORIZED_FIELD_NUMBER: _ClassVar[int]
    action: str
    is_authorized: bool
    def __init__(self, action: _Optional[str] = ..., is_authorized: bool = ...) -> None: ...

class ResourceTypePermissionValues(_message.Message):
    __slots__ = ["permission_values", "resource_type"]
    PERMISSION_VALUES_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    permission_values: _containers.RepeatedCompositeFieldContainer[PermissionValue]
    resource_type: _resource__client_pb2.ResourceType
    def __init__(self, resource_type: _Optional[_Union[_resource__client_pb2.ResourceType, str]] = ..., permission_values: _Optional[_Iterable[_Union[PermissionValue, _Mapping]]] = ...) -> None: ...

class ResourceTypePermissions(_message.Message):
    __slots__ = ["actions", "resource_type"]
    ACTIONS_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    actions: _containers.RepeatedScalarFieldContainer[str]
    resource_type: _resource__client_pb2.ResourceType
    def __init__(self, resource_type: _Optional[_Union[_resource__client_pb2.ResourceType, str]] = ..., actions: _Optional[_Iterable[str]] = ...) -> None: ...

class RoleDefinition(_message.Message):
    __slots__ = ["assignable_on_resource_types", "assignable_to_principal_types", "description", "id", "legacy_id", "name", "permissions"]
    ASSIGNABLE_ON_RESOURCE_TYPES_FIELD_NUMBER: _ClassVar[int]
    ASSIGNABLE_TO_PRINCIPAL_TYPES_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    LEGACY_ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
    assignable_on_resource_types: _containers.RepeatedScalarFieldContainer[_resource__client_pb2.ResourceType]
    assignable_to_principal_types: _containers.RepeatedScalarFieldContainer[_principal__client_pb2.PrincipalType]
    description: str
    id: str
    legacy_id: str
    name: str
    permissions: _containers.RepeatedCompositeFieldContainer[PermissionDefinition]
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., description: _Optional[str] = ..., assignable_on_resource_types: _Optional[_Iterable[_Union[_resource__client_pb2.ResourceType, str]]] = ..., assignable_to_principal_types: _Optional[_Iterable[_Union[_principal__client_pb2.PrincipalType, str]]] = ..., permissions: _Optional[_Iterable[_Union[PermissionDefinition, _Mapping]]] = ..., legacy_id: _Optional[str] = ...) -> None: ...

class UnassignRolesRequest(_message.Message):
    __slots__ = ["assignments"]
    ASSIGNMENTS_FIELD_NUMBER: _ClassVar[int]
    assignments: _containers.RepeatedCompositeFieldContainer[Assignment]
    def __init__(self, assignments: _Optional[_Iterable[_Union[Assignment, _Mapping]]] = ...) -> None: ...

class UnassignRolesResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...



from typing import Dict

from tecton_core import http
from tecton_core.secret_management import SecretCacheKey
from tecton_core.secret_management import SecretReference
from tecton_core.secret_management import SecretResolver


class MDSSecretResolver(SecretResolver):
    """Implementation of SecretsResolver that fetches secrets from the Tecton Secret Service and caches in-memory.

    Example:
        secret_resolver = SecretResolver(secrets_api_service_url="https://example.tecton.ai", api_key="my_api_key")
        db_password = secret_resolver.resolve(SecretReference(scope="prod", key="db_password"))
    """

    def __init__(self, secrets_api_service_url: str, api_key: str) -> None:
        if not api_key:
            msg = "Tecton secrets access API key not found."
            raise RuntimeError(msg)
        self.secrets_api_service_url = secrets_api_service_url
        self.secret_cache: Dict[SecretCacheKey, str] = {}
        self._headers = {"Authorization": f"Tecton-Key {api_key}"}

    def resolve(self, secret: SecretReference) -> str:
        cache_key = SecretCacheKey(secret.scope, secret.key)
        if cache_key not in self.secret_cache:
            self.secret_cache[cache_key] = self._fetch_secret(secret.scope, secret.key)
        return self.secret_cache[cache_key]

    def _fetch_secret(self, scope: str, key: str) -> str:
        """Fetch the secret value from the Secret Manager Service via HTTP."""
        # TODO: Switch to use PureHTTPStub, which allow service proto annotation once TEC-18444 is resolved.
        api_url = f"{self.secrets_api_service_url}/v1/secrets-service/scopes/{scope}/keys/{key}"
        response = http.session().get(api_url, verify=True, headers=self._headers)

        response.raise_for_status()
        response_json = response.json()
        if "value" in response_json:
            return response_json["value"]
        else:
            msg = f"Secret value not found when resolving secret for scope: {scope}, key: {key}"
            raise KeyError(msg)



from tecton_proto.args import data_source_config__client_pb2 as _data_source_config__client_pb2
from tecton_proto.args import feature_view__client_pb2 as _feature_view__client_pb2
from tecton_proto.args import user_defined_function__client_pb2 as _user_defined_function__client_pb2
from tecton_proto.args import virtual_data_source__client_pb2 as _virtual_data_source__client_pb2
from tecton_proto.common import schema__client_pb2 as _schema__client_pb2
from tecton_proto.common import spark_schema__client_pb2 as _spark_schema__client_pb2
from tecton_proto.data import feature_view__client_pb2 as _feature_view__client_pb2_1
from tecton_proto.data import hive_metastore__client_pb2 as _hive_metastore__client_pb2
from tecton_proto.data import transformation__client_pb2 as _transformation__client_pb2
from tecton_proto.data import virtual_data_source__client_pb2 as _virtual_data_source__client_pb2_1
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ExecuteRequest(_message.Message):
    __slots__ = ["envVars", "getBatchDataSourceFunctionSchema", "getFeatureViewSchema", "getFileSourceSchema", "getHiveTableSchema", "getKafkaSourceSchema", "getKinesisSourceSchema", "getMultipleDataSourceSchemasRequest", "getMultipleFeatureViewSchemasRequest", "getQueryPlanInfoForFeatureViewPipeline", "getRedshiftTableSchema", "getSnowflakeSchema", "getStreamDataSourceFunctionSchema", "getUnityTableSchema", "listHiveDatabases", "listHiveTableColumns", "listHiveTables"]
    class EnvVarsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ENVVARS_FIELD_NUMBER: _ClassVar[int]
    GETBATCHDATASOURCEFUNCTIONSCHEMA_FIELD_NUMBER: _ClassVar[int]
    GETFEATUREVIEWSCHEMA_FIELD_NUMBER: _ClassVar[int]
    GETFILESOURCESCHEMA_FIELD_NUMBER: _ClassVar[int]
    GETHIVETABLESCHEMA_FIELD_NUMBER: _ClassVar[int]
    GETKAFKASOURCESCHEMA_FIELD_NUMBER: _ClassVar[int]
    GETKINESISSOURCESCHEMA_FIELD_NUMBER: _ClassVar[int]
    GETMULTIPLEDATASOURCESCHEMASREQUEST_FIELD_NUMBER: _ClassVar[int]
    GETMULTIPLEFEATUREVIEWSCHEMASREQUEST_FIELD_NUMBER: _ClassVar[int]
    GETQUERYPLANINFOFORFEATUREVIEWPIPELINE_FIELD_NUMBER: _ClassVar[int]
    GETREDSHIFTTABLESCHEMA_FIELD_NUMBER: _ClassVar[int]
    GETSNOWFLAKESCHEMA_FIELD_NUMBER: _ClassVar[int]
    GETSTREAMDATASOURCEFUNCTIONSCHEMA_FIELD_NUMBER: _ClassVar[int]
    GETUNITYTABLESCHEMA_FIELD_NUMBER: _ClassVar[int]
    LISTHIVEDATABASES_FIELD_NUMBER: _ClassVar[int]
    LISTHIVETABLECOLUMNS_FIELD_NUMBER: _ClassVar[int]
    LISTHIVETABLES_FIELD_NUMBER: _ClassVar[int]
    envVars: _containers.ScalarMap[str, str]
    getBatchDataSourceFunctionSchema: GetBatchDataSourceFunctionSchema
    getFeatureViewSchema: GetFeatureViewSchema
    getFileSourceSchema: GetFileSourceSchema
    getHiveTableSchema: GetHiveTableSchema
    getKafkaSourceSchema: GetKafkaSourceSchema
    getKinesisSourceSchema: GetKinesisSourceSchema
    getMultipleDataSourceSchemasRequest: GetMultipleDataSourceSchemasRequest
    getMultipleFeatureViewSchemasRequest: GetMultipleFeatureViewSchemasRequest
    getQueryPlanInfoForFeatureViewPipeline: GetQueryPlanInfoForFeatureViewPipeline
    getRedshiftTableSchema: GetRedshiftTableSchema
    getSnowflakeSchema: GetSnowflakeSchema
    getStreamDataSourceFunctionSchema: GetStreamDataSourceFunctionSchema
    getUnityTableSchema: GetUnityTableSchema
    listHiveDatabases: ListHiveDatabases
    listHiveTableColumns: ListHiveTableColumns
    listHiveTables: ListHiveTables
    def __init__(self, getMultipleFeatureViewSchemasRequest: _Optional[_Union[GetMultipleFeatureViewSchemasRequest, _Mapping]] = ..., getMultipleDataSourceSchemasRequest: _Optional[_Union[GetMultipleDataSourceSchemasRequest, _Mapping]] = ..., getHiveTableSchema: _Optional[_Union[GetHiveTableSchema, _Mapping]] = ..., getRedshiftTableSchema: _Optional[_Union[GetRedshiftTableSchema, _Mapping]] = ..., getFileSourceSchema: _Optional[_Union[GetFileSourceSchema, _Mapping]] = ..., getKinesisSourceSchema: _Optional[_Union[GetKinesisSourceSchema, _Mapping]] = ..., getKafkaSourceSchema: _Optional[_Union[GetKafkaSourceSchema, _Mapping]] = ..., getFeatureViewSchema: _Optional[_Union[GetFeatureViewSchema, _Mapping]] = ..., getSnowflakeSchema: _Optional[_Union[GetSnowflakeSchema, _Mapping]] = ..., getBatchDataSourceFunctionSchema: _Optional[_Union[GetBatchDataSourceFunctionSchema, _Mapping]] = ..., getStreamDataSourceFunctionSchema: _Optional[_Union[GetStreamDataSourceFunctionSchema, _Mapping]] = ..., getUnityTableSchema: _Optional[_Union[GetUnityTableSchema, _Mapping]] = ..., listHiveDatabases: _Optional[_Union[ListHiveDatabases, _Mapping]] = ..., listHiveTables: _Optional[_Union[ListHiveTables, _Mapping]] = ..., listHiveTableColumns: _Optional[_Union[ListHiveTableColumns, _Mapping]] = ..., getQueryPlanInfoForFeatureViewPipeline: _Optional[_Union[GetQueryPlanInfoForFeatureViewPipeline, _Mapping]] = ..., envVars: _Optional[_Mapping[str, str]] = ...) -> None: ...

class ExecuteResult(_message.Message):
    __slots__ = ["listHiveResult", "multipleDataSourceSchemaResponse", "multipleFeatureViewSchemaResponse", "queryPlanInfo", "schema", "sparkSchema", "uncaughtError", "validationError"]
    LISTHIVERESULT_FIELD_NUMBER: _ClassVar[int]
    MULTIPLEDATASOURCESCHEMARESPONSE_FIELD_NUMBER: _ClassVar[int]
    MULTIPLEFEATUREVIEWSCHEMARESPONSE_FIELD_NUMBER: _ClassVar[int]
    QUERYPLANINFO_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    SPARKSCHEMA_FIELD_NUMBER: _ClassVar[int]
    UNCAUGHTERROR_FIELD_NUMBER: _ClassVar[int]
    VALIDATIONERROR_FIELD_NUMBER: _ClassVar[int]
    listHiveResult: _hive_metastore__client_pb2.ListHiveResult
    multipleDataSourceSchemaResponse: GetMultipleDataSourceSchemasResponse
    multipleFeatureViewSchemaResponse: GetMultipleFeatureViewSchemasResponse
    queryPlanInfo: QueryPlanInfo
    schema: _schema__client_pb2.Schema
    sparkSchema: _spark_schema__client_pb2.SparkSchema
    uncaughtError: str
    validationError: str
    def __init__(self, uncaughtError: _Optional[str] = ..., validationError: _Optional[str] = ..., sparkSchema: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ..., queryPlanInfo: _Optional[_Union[QueryPlanInfo, _Mapping]] = ..., schema: _Optional[_Union[_schema__client_pb2.Schema, _Mapping]] = ..., listHiveResult: _Optional[_Union[_hive_metastore__client_pb2.ListHiveResult, _Mapping]] = ..., multipleFeatureViewSchemaResponse: _Optional[_Union[GetMultipleFeatureViewSchemasResponse, _Mapping]] = ..., multipleDataSourceSchemaResponse: _Optional[_Union[GetMultipleDataSourceSchemasResponse, _Mapping]] = ...) -> None: ...

class FeatureViewSchemaRequest(_message.Message):
    __slots__ = ["feature_view", "join_keys", "temporal_aggregate"]
    FEATURE_VIEW_FIELD_NUMBER: _ClassVar[int]
    JOIN_KEYS_FIELD_NUMBER: _ClassVar[int]
    TEMPORAL_AGGREGATE_FIELD_NUMBER: _ClassVar[int]
    feature_view: _feature_view__client_pb2.FeatureViewArgs
    join_keys: _containers.RepeatedScalarFieldContainer[str]
    temporal_aggregate: _feature_view__client_pb2_1.TemporalAggregate
    def __init__(self, feature_view: _Optional[_Union[_feature_view__client_pb2.FeatureViewArgs, _Mapping]] = ..., join_keys: _Optional[_Iterable[str]] = ..., temporal_aggregate: _Optional[_Union[_feature_view__client_pb2_1.TemporalAggregate, _Mapping]] = ...) -> None: ...

class FeatureViewSchemaResponse(_message.Message):
    __slots__ = ["timestamp_key", "view_schema"]
    TIMESTAMP_KEY_FIELD_NUMBER: _ClassVar[int]
    VIEW_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    timestamp_key: str
    view_schema: _schema__client_pb2.Schema
    def __init__(self, view_schema: _Optional[_Union[_schema__client_pb2.Schema, _Mapping]] = ..., timestamp_key: _Optional[str] = ...) -> None: ...

class GetBatchDataSourceFunctionSchema(_message.Message):
    __slots__ = ["function", "supports_time_filtering"]
    FUNCTION_FIELD_NUMBER: _ClassVar[int]
    SUPPORTS_TIME_FILTERING_FIELD_NUMBER: _ClassVar[int]
    function: _user_defined_function__client_pb2.UserDefinedFunction
    supports_time_filtering: bool
    def __init__(self, function: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ..., supports_time_filtering: bool = ...) -> None: ...

class GetFeatureViewSchema(_message.Message):
    __slots__ = ["feature_view", "transformations", "virtual_data_sources"]
    FEATURE_VIEW_FIELD_NUMBER: _ClassVar[int]
    TRANSFORMATIONS_FIELD_NUMBER: _ClassVar[int]
    VIRTUAL_DATA_SOURCES_FIELD_NUMBER: _ClassVar[int]
    feature_view: _feature_view__client_pb2.FeatureViewArgs
    transformations: _containers.RepeatedCompositeFieldContainer[_transformation__client_pb2.Transformation]
    virtual_data_sources: _containers.RepeatedCompositeFieldContainer[_virtual_data_source__client_pb2_1.VirtualDataSource]
    def __init__(self, virtual_data_sources: _Optional[_Iterable[_Union[_virtual_data_source__client_pb2_1.VirtualDataSource, _Mapping]]] = ..., transformations: _Optional[_Iterable[_Union[_transformation__client_pb2.Transformation, _Mapping]]] = ..., feature_view: _Optional[_Union[_feature_view__client_pb2.FeatureViewArgs, _Mapping]] = ...) -> None: ...

class GetFileSourceSchema(_message.Message):
    __slots__ = ["convertToGlueFormat", "fileFormat", "rawBatchTranslator", "schemaOverride", "schemaUri", "timestampColumn", "timestampFormat", "uri"]
    CONVERTTOGLUEFORMAT_FIELD_NUMBER: _ClassVar[int]
    FILEFORMAT_FIELD_NUMBER: _ClassVar[int]
    RAWBATCHTRANSLATOR_FIELD_NUMBER: _ClassVar[int]
    SCHEMAOVERRIDE_FIELD_NUMBER: _ClassVar[int]
    SCHEMAURI_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMPCOLUMN_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMPFORMAT_FIELD_NUMBER: _ClassVar[int]
    URI_FIELD_NUMBER: _ClassVar[int]
    convertToGlueFormat: bool
    fileFormat: str
    rawBatchTranslator: _user_defined_function__client_pb2.UserDefinedFunction
    schemaOverride: _spark_schema__client_pb2.SparkSchema
    schemaUri: str
    timestampColumn: str
    timestampFormat: str
    uri: str
    def __init__(self, uri: _Optional[str] = ..., fileFormat: _Optional[str] = ..., convertToGlueFormat: bool = ..., rawBatchTranslator: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ..., schemaUri: _Optional[str] = ..., timestampColumn: _Optional[str] = ..., timestampFormat: _Optional[str] = ..., schemaOverride: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ...) -> None: ...

class GetHiveTableSchema(_message.Message):
    __slots__ = ["database", "rawBatchTranslator", "table", "timestampColumn", "timestampFormat"]
    DATABASE_FIELD_NUMBER: _ClassVar[int]
    RAWBATCHTRANSLATOR_FIELD_NUMBER: _ClassVar[int]
    TABLE_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMPCOLUMN_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMPFORMAT_FIELD_NUMBER: _ClassVar[int]
    database: str
    rawBatchTranslator: _user_defined_function__client_pb2.UserDefinedFunction
    table: str
    timestampColumn: str
    timestampFormat: str
    def __init__(self, database: _Optional[str] = ..., table: _Optional[str] = ..., timestampColumn: _Optional[str] = ..., timestampFormat: _Optional[str] = ..., rawBatchTranslator: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ...) -> None: ...

class GetKafkaSourceSchema(_message.Message):
    __slots__ = ["rawStreamTranslator", "ssl_keystore_location", "ssl_keystore_password_secret_id"]
    RAWSTREAMTRANSLATOR_FIELD_NUMBER: _ClassVar[int]
    SSL_KEYSTORE_LOCATION_FIELD_NUMBER: _ClassVar[int]
    SSL_KEYSTORE_PASSWORD_SECRET_ID_FIELD_NUMBER: _ClassVar[int]
    rawStreamTranslator: _user_defined_function__client_pb2.UserDefinedFunction
    ssl_keystore_location: str
    ssl_keystore_password_secret_id: str
    def __init__(self, rawStreamTranslator: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ..., ssl_keystore_location: _Optional[str] = ..., ssl_keystore_password_secret_id: _Optional[str] = ...) -> None: ...

class GetKinesisSourceSchema(_message.Message):
    __slots__ = ["rawStreamTranslator", "streamName"]
    RAWSTREAMTRANSLATOR_FIELD_NUMBER: _ClassVar[int]
    STREAMNAME_FIELD_NUMBER: _ClassVar[int]
    rawStreamTranslator: _user_defined_function__client_pb2.UserDefinedFunction
    streamName: str
    def __init__(self, streamName: _Optional[str] = ..., rawStreamTranslator: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ...) -> None: ...

class GetMultipleDataSourceSchemasRequest(_message.Message):
    __slots__ = ["data_source_args"]
    DATA_SOURCE_ARGS_FIELD_NUMBER: _ClassVar[int]
    data_source_args: _containers.RepeatedCompositeFieldContainer[_virtual_data_source__client_pb2.VirtualDataSourceArgs]
    def __init__(self, data_source_args: _Optional[_Iterable[_Union[_virtual_data_source__client_pb2.VirtualDataSourceArgs, _Mapping]]] = ...) -> None: ...

class GetMultipleDataSourceSchemasResponse(_message.Message):
    __slots__ = ["data_source_responses"]
    class DataSourceResponsesEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: SparkSchemas
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[SparkSchemas, _Mapping]] = ...) -> None: ...
    DATA_SOURCE_RESPONSES_FIELD_NUMBER: _ClassVar[int]
    data_source_responses: _containers.MessageMap[str, SparkSchemas]
    def __init__(self, data_source_responses: _Optional[_Mapping[str, SparkSchemas]] = ...) -> None: ...

class GetMultipleFeatureViewSchemasRequest(_message.Message):
    __slots__ = ["feature_view_requests", "transformations", "virtual_data_sources"]
    FEATURE_VIEW_REQUESTS_FIELD_NUMBER: _ClassVar[int]
    TRANSFORMATIONS_FIELD_NUMBER: _ClassVar[int]
    VIRTUAL_DATA_SOURCES_FIELD_NUMBER: _ClassVar[int]
    feature_view_requests: _containers.RepeatedCompositeFieldContainer[FeatureViewSchemaRequest]
    transformations: _containers.RepeatedCompositeFieldContainer[_transformation__client_pb2.Transformation]
    virtual_data_sources: _containers.RepeatedCompositeFieldContainer[_virtual_data_source__client_pb2_1.VirtualDataSource]
    def __init__(self, virtual_data_sources: _Optional[_Iterable[_Union[_virtual_data_source__client_pb2_1.VirtualDataSource, _Mapping]]] = ..., transformations: _Optional[_Iterable[_Union[_transformation__client_pb2.Transformation, _Mapping]]] = ..., feature_view_requests: _Optional[_Iterable[_Union[FeatureViewSchemaRequest, _Mapping]]] = ...) -> None: ...

class GetMultipleFeatureViewSchemasResponse(_message.Message):
    __slots__ = ["feature_view_responses"]
    class FeatureViewResponsesEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: FeatureViewSchemaResponse
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[FeatureViewSchemaResponse, _Mapping]] = ...) -> None: ...
    FEATURE_VIEW_RESPONSES_FIELD_NUMBER: _ClassVar[int]
    feature_view_responses: _containers.MessageMap[str, FeatureViewSchemaResponse]
    def __init__(self, feature_view_responses: _Optional[_Mapping[str, FeatureViewSchemaResponse]] = ...) -> None: ...

class GetQueryPlanInfoForFeatureViewPipeline(_message.Message):
    __slots__ = ["feature_view", "transformations", "virtual_data_sources"]
    FEATURE_VIEW_FIELD_NUMBER: _ClassVar[int]
    TRANSFORMATIONS_FIELD_NUMBER: _ClassVar[int]
    VIRTUAL_DATA_SOURCES_FIELD_NUMBER: _ClassVar[int]
    feature_view: _feature_view__client_pb2.FeatureViewArgs
    transformations: _containers.RepeatedCompositeFieldContainer[_transformation__client_pb2.Transformation]
    virtual_data_sources: _containers.RepeatedCompositeFieldContainer[_virtual_data_source__client_pb2_1.VirtualDataSource]
    def __init__(self, virtual_data_sources: _Optional[_Iterable[_Union[_virtual_data_source__client_pb2_1.VirtualDataSource, _Mapping]]] = ..., transformations: _Optional[_Iterable[_Union[_transformation__client_pb2.Transformation, _Mapping]]] = ..., feature_view: _Optional[_Union[_feature_view__client_pb2.FeatureViewArgs, _Mapping]] = ...) -> None: ...

class GetRedshiftTableSchema(_message.Message):
    __slots__ = ["endpoint", "query", "rawBatchTranslator", "table", "temp_s3_dir"]
    ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    QUERY_FIELD_NUMBER: _ClassVar[int]
    RAWBATCHTRANSLATOR_FIELD_NUMBER: _ClassVar[int]
    TABLE_FIELD_NUMBER: _ClassVar[int]
    TEMP_S3_DIR_FIELD_NUMBER: _ClassVar[int]
    endpoint: str
    query: str
    rawBatchTranslator: _user_defined_function__client_pb2.UserDefinedFunction
    table: str
    temp_s3_dir: str
    def __init__(self, endpoint: _Optional[str] = ..., table: _Optional[str] = ..., query: _Optional[str] = ..., rawBatchTranslator: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ..., temp_s3_dir: _Optional[str] = ...) -> None: ...

class GetSnowflakeSchema(_message.Message):
    __slots__ = ["database", "post_processor", "query", "role", "schema", "table", "url", "warehouse"]
    DATABASE_FIELD_NUMBER: _ClassVar[int]
    POST_PROCESSOR_FIELD_NUMBER: _ClassVar[int]
    QUERY_FIELD_NUMBER: _ClassVar[int]
    ROLE_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    TABLE_FIELD_NUMBER: _ClassVar[int]
    URL_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_FIELD_NUMBER: _ClassVar[int]
    database: str
    post_processor: _user_defined_function__client_pb2.UserDefinedFunction
    query: str
    role: str
    schema: str
    table: str
    url: str
    warehouse: str
    def __init__(self, url: _Optional[str] = ..., role: _Optional[str] = ..., database: _Optional[str] = ..., schema: _Optional[str] = ..., warehouse: _Optional[str] = ..., table: _Optional[str] = ..., query: _Optional[str] = ..., post_processor: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ...) -> None: ...

class GetStreamDataSourceFunctionSchema(_message.Message):
    __slots__ = ["function"]
    FUNCTION_FIELD_NUMBER: _ClassVar[int]
    function: _user_defined_function__client_pb2.UserDefinedFunction
    def __init__(self, function: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ...) -> None: ...

class GetUnityTableSchema(_message.Message):
    __slots__ = ["catalog", "rawBatchTranslator", "schema", "table", "timestampColumn", "timestampFormat"]
    CATALOG_FIELD_NUMBER: _ClassVar[int]
    RAWBATCHTRANSLATOR_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    TABLE_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMPCOLUMN_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMPFORMAT_FIELD_NUMBER: _ClassVar[int]
    catalog: str
    rawBatchTranslator: _user_defined_function__client_pb2.UserDefinedFunction
    schema: str
    table: str
    timestampColumn: str
    timestampFormat: str
    def __init__(self, catalog: _Optional[str] = ..., schema: _Optional[str] = ..., table: _Optional[str] = ..., timestampColumn: _Optional[str] = ..., timestampFormat: _Optional[str] = ..., rawBatchTranslator: _Optional[_Union[_user_defined_function__client_pb2.UserDefinedFunction, _Mapping]] = ...) -> None: ...

class ListHiveDatabases(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ListHiveTableColumns(_message.Message):
    __slots__ = ["database", "table"]
    DATABASE_FIELD_NUMBER: _ClassVar[int]
    TABLE_FIELD_NUMBER: _ClassVar[int]
    database: str
    table: str
    def __init__(self, database: _Optional[str] = ..., table: _Optional[str] = ...) -> None: ...

class ListHiveTables(_message.Message):
    __slots__ = ["database"]
    DATABASE_FIELD_NUMBER: _ClassVar[int]
    database: str
    def __init__(self, database: _Optional[str] = ...) -> None: ...

class QueryPlanInfo(_message.Message):
    __slots__ = ["has_aggregations", "has_joins"]
    HAS_AGGREGATIONS_FIELD_NUMBER: _ClassVar[int]
    HAS_JOINS_FIELD_NUMBER: _ClassVar[int]
    has_aggregations: bool
    has_joins: bool
    def __init__(self, has_joins: bool = ..., has_aggregations: bool = ...) -> None: ...

class SparkSchemas(_message.Message):
    __slots__ = ["batchSchema", "streamSchema"]
    BATCHSCHEMA_FIELD_NUMBER: _ClassVar[int]
    STREAMSCHEMA_FIELD_NUMBER: _ClassVar[int]
    batchSchema: _spark_schema__client_pb2.SparkSchema
    streamSchema: _spark_schema__client_pb2.SparkSchema
    def __init__(self, batchSchema: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ..., streamSchema: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ...) -> None: ...
